@@ -3,8 +3,10 @@ import os, stat, time, re
 from allmydata.interfaces import RIStorageServer
 from allmydata import node
 
+from zope.interface import implements
 from twisted.internet import reactor
 from twisted.application.internet import TimerService
+from foolscap import Referenceable
 from foolscap.logging import log
 
 import allmydata
@@ -18,10 +20,10 @@ from allmydata.introducer import IntroducerClient
 from allmydata.util import hashutil, base32, testutil
 from allmydata.filenode import FileNode
 from allmydata.dirnode import NewDirectoryNode
-from allmydata.mutable import MutableFileNode
+from allmydata.mutable import MutableFileNode, MutableWatcher
 from allmydata.stats import StatsProvider
 from allmydata.interfaces import IURI, INewDirectoryURI, \
-     IReadonlyNewDirectoryURI, IFileURI, IMutableFileURI
+     IReadonlyNewDirectoryURI, IFileURI, IMutableFileURI, RIStubClient
 
 KiB=1024
 MiB=1024*KiB
@@ -29,6 +31,9 @@ GiB=1024*MiB
 TiB=1024*GiB
 PiB=1024*TiB
 
+class StubClient(Referenceable):
+    implements(RIStubClient)
+
 class Client(node.Node, testutil.PollMixin):
     PORTNUMFILE = "client.port"
     STOREDIR = 'storage'
@@ -46,7 +51,7 @@ class Client(node.Node, testutil.PollMixin):
     DEFAULT_ENCODING_PARAMETERS = {"k": 3,
                                    "happy": 7,
                                    "n": 10,
-                                   "max_segment_size": 1*MiB,
+                                   "max_segment_size": 128*KiB,
                                    }
 
     def __init__(self, basedir="."):
@@ -63,10 +68,7 @@ class Client(node.Node, testutil.PollMixin):
         run_helper = self.get_config("run_helper")
         if run_helper:
             self.init_helper()
-        helper_furl = self.get_config("helper.furl")
-        self.add_service(Uploader(helper_furl))
-        self.add_service(Downloader())
-        self.add_service(Checker())
+        self.init_client()
         # ControlServer and Helper are attached after Tub startup
 
         hotline_file = os.path.join(self.basedir,
@@ -147,6 +149,24 @@ class Client(node.Node, testutil.PollMixin):
         d.addCallback(_publish)
         d.addErrback(log.err, facility="tahoe.init", level=log.BAD)
 
+    def init_client(self):
+        helper_furl = self.get_config("helper.furl")
+        self.add_service(Uploader(helper_furl))
+        self.add_service(Downloader())
+        self.add_service(Checker())
+        self.add_service(MutableWatcher())
+        def _publish(res):
+            # we publish an empty object so that the introducer can count how
+            # many clients are connected and see what versions they're
+            # running.
+            sc = StubClient()
+            furl = self.tub.registerReference(sc)
+            ri_name = RIStubClient.__remote_name__
+            self.introducer_client.publish(furl, "stub_client", ri_name)
+        d = self.when_tub_ready()
+        d.addCallback(_publish)
+        d.addErrback(log.err, facility="tahoe.init", level=log.BAD)
+
     def init_control(self):
         d = self.when_tub_ready()
         def _publish(res):
@@ -207,19 +227,6 @@ class Client(node.Node, testutil.PollMixin):
 
     def get_encoding_parameters(self):
         return self.DEFAULT_ENCODING_PARAMETERS
-        p = self.introducer_client.encoding_parameters # a tuple
-        # TODO: make the 0.7.1 introducer publish a dict instead of a tuple
-        params = {"k": p[0],
-                  "happy": p[1],
-                  "n": p[2],
-                  }
-        if len(p) == 3:
-            # TODO: compatibility with 0.7.0 Introducer that doesn't specify
-            # segment_size
-            self.log("Introducer didn't provide max_segment_size, using 1MiB",
-                     level=log.UNUSUAL)
-            params["max_segment_size"] = 1*MiB
-        return params
 
     def connected_to_introducer(self):
         if self.introducer_client:
@@ -264,6 +271,11 @@ class Client(node.Node, testutil.PollMixin):
         assert IMutableFileURI.providedBy(u), u
         return MutableFileNode(self).init_from_uri(u)
 
+    def notify_publish(self, p):
+        self.getServiceNamed("mutable-watcher").notify_publish(p)
+    def notify_retrieve(self, r):
+        self.getServiceNamed("mutable-watcher").notify_retrieve(r)
+
     def create_empty_dirnode(self):
         n = NewDirectoryNode(self)
         d = n.create()
@@ -280,10 +292,43 @@ class Client(node.Node, testutil.PollMixin):
         uploader = self.getServiceNamed("uploader")
         return uploader.upload(uploadable)
 
-    def list_uploads(self):
-        uploader = self.getServiceNamed("uploader")
-        return uploader.list_uploads()
 
-    def list_downloads(self):
+    def list_all_uploads(self):
+        uploader = self.getServiceNamed("uploader")
+        return uploader.list_all_uploads()
+    def list_active_uploads(self):
+        uploader = self.getServiceNamed("uploader")
+        return uploader.list_active_uploads()
+    def list_recent_uploads(self):
+        uploader = self.getServiceNamed("uploader")
+        return uploader.list_recent_uploads()
+
+    def list_all_downloads(self):
         downloader = self.getServiceNamed("downloader")
-        return downloader.list_downloads()
+        return downloader.list_all_downloads()
+    def list_active_downloads(self):
+        downloader = self.getServiceNamed("downloader")
+        return downloader.list_active_downloads()
+    def list_recent_downloads(self):
+        downloader = self.getServiceNamed("downloader")
+        return downloader.list_recent_downloads()
+
+    def list_all_publish(self):
+        watcher = self.getServiceNamed("mutable-watcher")
+        return watcher.list_all_publish()
+    def list_active_publish(self):
+        watcher = self.getServiceNamed("mutable-watcher")
+        return watcher.list_active_publish()
+    def list_recent_publish(self):
+        watcher = self.getServiceNamed("mutable-watcher")
+        return watcher.list_recent_publish()
+
+    def list_all_retrieve(self):
+        watcher = self.getServiceNamed("mutable-watcher")
+        return watcher.list_all_retrieve()
+    def list_active_retrieve(self):
+        watcher = self.getServiceNamed("mutable-watcher")
+        return watcher.list_active_retrieve()
+    def list_recent_retrieve(self):
+        watcher = self.getServiceNamed("mutable-watcher")
+        return watcher.list_recent_retrieve()
