@@ -1,5 +1,5 @@
 
-import os, random, weakref
+import os, random, weakref, itertools, time
 from zope.interface import implements
 from twisted.internet import defer
 from twisted.internet.interfaces import IPushProducer, IConsumer
@@ -10,7 +10,7 @@ from allmydata.util import base32, mathutil, hashutil, log
 from allmydata.util.assertutil import _assert
 from allmydata import codec, hashtree, storage, uri
 from allmydata.interfaces import IDownloadTarget, IDownloader, IFileURI, \
-     IDownloadStatus
+     IDownloadStatus, IDownloadResults
 from allmydata.encode import NotEnoughPeersError
 from pycryptopp.cipher.aes import AES
 
@@ -28,6 +28,16 @@ class BadCrypttextHashValue(Exception):
 class DownloadStopped(Exception):
     pass
 
+class DownloadResults:
+    implements(IDownloadResults)
+
+    def __init__(self):
+        self.servers_used = set()
+        self.server_problems = {}
+        self.servermap = {}
+        self.timings = {}
+        self.file_size = None
+
 class Output:
     def __init__(self, downloadable, key, total_length, log_parent,
                  download_status):
@@ -237,10 +247,11 @@ class BlockDownloader:
     I am a child of the SegmentDownloader.
     """
 
-    def __init__(self, vbucket, blocknum, parent):
+    def __init__(self, vbucket, blocknum, parent, results):
         self.vbucket = vbucket
         self.blocknum = blocknum
         self.parent = parent
+        self.results = results
         self._log_number = self.parent.log("starting block %d" % blocknum)
 
     def log(self, msg, parent=None):
@@ -250,18 +261,28 @@ class BlockDownloader:
 
     def start(self, segnum):
         lognum = self.log("get_block(segnum=%d)" % segnum)
+        started = time.time()
         d = self.vbucket.get_block(segnum)
         d.addCallbacks(self._hold_block, self._got_block_error,
-                       callbackArgs=(lognum,), errbackArgs=(lognum,))
+                       callbackArgs=(started, lognum,), errbackArgs=(lognum,))
         return d
 
-    def _hold_block(self, data, lognum):
+    def _hold_block(self, data, started, lognum):
+        if self.results:
+            elapsed = time.time() - started
+            peerid = self.vbucket.bucket.get_peerid()
+            if peerid not in self.results.timings["fetch_per_server"]:
+                self.results.timings["fetch_per_server"][peerid] = []
+            self.results.timings["fetch_per_server"][peerid].append(elapsed)
         self.log("got block", parent=lognum)
         self.parent.hold_block(self.blocknum, data)
 
     def _got_block_error(self, f, lognum):
         self.log("BlockDownloader[%d] got error: %s" % (self.blocknum, f),
                  parent=lognum)
+        if self.results:
+            peerid = self.vbucket.bucket.get_peerid()
+            self.results.server_problems[peerid] = str(f)
         self.parent.bucket_failed(self.vbucket)
 
 class SegmentDownloader:
@@ -271,11 +292,12 @@ class SegmentDownloader:
     I am a child of the FileDownloader.
     """
 
-    def __init__(self, parent, segmentnumber, needed_shares):
+    def __init__(self, parent, segmentnumber, needed_shares, results):
         self.parent = parent
         self.segmentnumber = segmentnumber
         self.needed_blocks = needed_shares
         self.blocks = {} # k: blocknum, v: data
+        self.results = results
         self._log_number = self.parent.log("starting segment %d" %
                                            segmentnumber)
 
@@ -314,8 +336,10 @@ class SegmentDownloader:
         # through it.
         downloaders = []
         for blocknum, vbucket in active_buckets.iteritems():
-            bd = BlockDownloader(vbucket, blocknum, self)
+            bd = BlockDownloader(vbucket, blocknum, self, self.results)
             downloaders.append(bd)
+            if self.results:
+                self.results.servers_used.add(vbucket.bucket.get_peerid())
         l = [bd.start(self.segmentnumber) for bd in downloaders]
         return defer.DeferredList(l, fireOnOneErrback=True)
 
@@ -327,6 +351,7 @@ class SegmentDownloader:
 
 class DownloadStatus:
     implements(IDownloadStatus)
+    statusid_counter = itertools.count(0)
 
     def __init__(self):
         self.storage_index = None
@@ -336,7 +361,13 @@ class DownloadStatus:
         self.progress = 0.0
         self.paused = False
         self.stopped = False
+        self.active = True
+        self.results = None
+        self.counter = self.statusid_counter.next()
+        self.started = time.time()
 
+    def get_started(self):
+        return self.started
     def get_storage_index(self):
         return self.storage_index
     def get_size(self):
@@ -352,6 +383,12 @@ class DownloadStatus:
         return status
     def get_progress(self):
         return self.progress
+    def get_active(self):
+        return self.active
+    def get_results(self):
+        return self.results
+    def get_counter(self):
+        return self.counter
 
     def set_storage_index(self, si):
         self.storage_index = si
@@ -367,7 +404,10 @@ class DownloadStatus:
         self.stopped = stopped
     def set_progress(self, value):
         self.progress = value
-
+    def set_active(self, value):
+        self.active = value
+    def set_results(self, value):
+        self.results = value
 
 class FileDownloader:
     implements(IPushProducer)
@@ -385,10 +425,25 @@ class FileDownloader:
         self._size = u.size
         self._num_needed_shares = u.needed_shares
 
+        self._si_s = storage.si_b2a(self._storage_index)
         self.init_logging()
 
+        self._started = time.time()
         self._status = s = DownloadStatus()
         s.set_status("Starting")
+        s.set_storage_index(self._storage_index)
+        s.set_size(self._size)
+        s.set_helper(False)
+        s.set_active(True)
+
+        self._results = DownloadResults()
+        s.set_results(self._results)
+        self._results.file_size = self._size
+        self._results.timings["servers_peer_selection"] = {}
+        self._results.timings["fetch_per_server"] = {}
+        self._results.timings["cumulative_fetch"] = 0.0
+        self._results.timings["cumulative_decode"] = 0.0
+        self._results.timings["cumulative_decrypt"] = 0.0
 
         if IConsumer.providedBy(downloadable):
             downloadable.registerProducer(self, True)
@@ -445,6 +500,7 @@ class FileDownloader:
         self._stopped = True
         if self._status:
             self._status.set_stopped(True)
+            self._status.set_active(False)
 
     def start(self):
         self.log("starting download")
@@ -462,6 +518,8 @@ class FileDownloader:
         def _finished(res):
             if self._status:
                 self._status.set_status("Finished")
+                self._status.set_active(False)
+                self._status.set_paused(False)
             if IConsumer.providedBy(self._downloadable):
                 self._downloadable.unregisterProducer()
             return res
@@ -469,6 +527,7 @@ class FileDownloader:
         def _failed(why):
             if self._status:
                 self._status.set_status("Failed")
+                self._status.set_active(False)
             self._output.fail(why)
             return why
         d.addErrback(_failed)
@@ -480,7 +539,8 @@ class FileDownloader:
         for (peerid,ss) in self._client.get_permuted_peers("storage",
                                                            self._storage_index):
             d = ss.callRemote("get_buckets", self._storage_index)
-            d.addCallbacks(self._got_response, self._got_error)
+            d.addCallbacks(self._got_response, self._got_error,
+                           callbackArgs=(peerid,))
             dl.append(d)
         self._responses_received = 0
         self._queries_sent = len(dl)
@@ -490,16 +550,23 @@ class FileDownloader:
                                      self._queries_sent))
         return defer.DeferredList(dl)
 
-    def _got_response(self, buckets):
+    def _got_response(self, buckets, peerid):
         self._responses_received += 1
+        if self._results:
+            elapsed = time.time() - self._started
+            self._results.timings["servers_peer_selection"][peerid] = elapsed
         if self._status:
             self._status.set_status("Locating Shares (%d/%d)" %
                                     (self._responses_received,
                                      self._queries_sent))
         for sharenum, bucket in buckets.iteritems():
-            b = storage.ReadBucketProxy(bucket)
+            b = storage.ReadBucketProxy(bucket, peerid, self._si_s)
             self.add_share_bucket(sharenum, b)
             self._uri_extension_sources.append(b)
+            if self._results:
+                if peerid not in self._results.servermap:
+                    self._results.servermap[peerid] = set()
+                self._results.servermap[peerid].add(sharenum)
 
     def add_share_bucket(self, sharenum, bucket):
         # this is split out for the benefit of test_encode.py
@@ -521,6 +588,10 @@ class FileDownloader:
             del self._share_vbuckets[shnum]
 
     def _got_all_shareholders(self, res):
+        if self._results:
+            now = time.time()
+            self._results.timings["peer_selection"] = now - self._started
+
         if len(self._share_buckets) < self._num_needed_shares:
             raise NotEnoughPeersError
 
@@ -540,12 +611,17 @@ class FileDownloader:
         if self._status:
             self._status.set_status("Obtaining URI Extension")
 
+        self._uri_extension_fetch_started = time.time()
         def _validate(proposal, bucket):
             h = hashutil.uri_extension_hash(proposal)
             if h != self._uri_extension_hash:
                 self._fetch_failures["uri_extension"] += 1
                 msg = ("The copy of uri_extension we received from "
-                       "%s was bad" % bucket)
+                       "%s was bad: wanted %s, got %s" %
+                       (bucket,
+                        base32.b2a(self._uri_extension_hash),
+                        base32.b2a(h)))
+                self.log(msg, level=log.SCARY)
                 raise BadURIExtensionHashValue(msg)
             return self._unpack_uri_extension_data(proposal)
         return self._obtain_validated_thing(None,
@@ -565,7 +641,8 @@ class FileDownloader:
         d.addCallback(lambda res: getattr(bucket, methname)(*args))
         d.addCallback(validatorfunc, bucket)
         def _bad(f):
-            self.log("WEIRD: %s from vbucket %s failed: %s" % (name, bucket, f))
+            self.log("%s from vbucket %s failed:" % (name, bucket),
+                     failure=f, level=log.WEIRD)
             if not sources:
                 raise NotEnoughPeersError("ran out of peers, last error was %s"
                                           % (f,))
@@ -576,6 +653,10 @@ class FileDownloader:
         return d
 
     def _got_uri_extension(self, uri_extension_data):
+        if self._results:
+            elapsed = time.time() - self._uri_extension_fetch_started
+            self._results.timings["uri_extension"] = elapsed
+
         d = self._uri_extension_data = uri_extension_data
 
         self._codec = codec.get_decoder_by_name(d['codec_name'])
@@ -598,6 +679,7 @@ class FileDownloader:
         self._share_hashtree.set_hashes({0: self._roothash})
 
     def _get_hashtrees(self, res):
+        self._get_hashtrees_started = time.time()
         if self._status:
             self._status.set_status("Retrieving Hash Trees")
         d = self._get_plaintext_hashtrees()
@@ -656,7 +738,9 @@ class FileDownloader:
     def _setup_hashtrees(self, res):
         self._output.setup_hashtrees(self._plaintext_hashtree,
                                      self._crypttext_hashtree)
-
+        if self._results:
+            elapsed = time.time() - self._get_hashtrees_started
+            self._results.timings["hashtrees"] = elapsed
 
     def _create_validated_buckets(self, ignored=None):
         self._share_vbuckets = {}
@@ -696,6 +780,8 @@ class FileDownloader:
         # RIBucketReader references.
         self.active_buckets = {} # k: shnum, v: ValidatedBucket instance
 
+        self._started_fetching = time.time()
+
         d = defer.succeed(None)
         for segnum in range(self._total_segments-1):
             d.addCallback(self._download_segment, segnum)
@@ -717,22 +803,41 @@ class FileDownloader:
     def _download_segment(self, res, segnum):
         if self._status:
             self._status.set_status("Downloading segment %d of %d" %
-                                    (segnum, self._total_segments))
+                                    (segnum+1, self._total_segments))
         self.log("downloading seg#%d of %d (%d%%)"
                  % (segnum, self._total_segments,
                     100.0 * segnum / self._total_segments))
         # memory footprint: when the SegmentDownloader finishes pulling down
         # all shares, we have 1*segment_size of usage.
-        segmentdler = SegmentDownloader(self, segnum, self._num_needed_shares)
+        segmentdler = SegmentDownloader(self, segnum, self._num_needed_shares,
+                                        self._results)
+        started = time.time()
         d = segmentdler.start()
+        def _finished_fetching(res):
+            elapsed = time.time() - started
+            self._results.timings["cumulative_fetch"] += elapsed
+            return res
+        if self._results:
+            d.addCallback(_finished_fetching)
         # pause before using more memory
         d.addCallback(self._check_for_pause)
         # while the codec does its job, we hit 2*segment_size
+        def _started_decode(res):
+            self._started_decode = time.time()
+            return res
+        if self._results:
+            d.addCallback(_started_decode)
         d.addCallback(lambda (shares, shareids):
                       self._codec.decode(shares, shareids))
         # once the codec is done, we drop back to 1*segment_size, because
         # 'shares' goes out of scope. The memory usage is all in the
         # plaintext now, spread out into a bunch of tiny buffers.
+        def _finished_decode(res):
+            elapsed = time.time() - self._started_decode
+            self._results.timings["cumulative_decode"] += elapsed
+            return res
+        if self._results:
+            d.addCallback(_finished_decode)
 
         # pause/check-for-stop just before writing, to honor stopProducing
         d.addCallback(self._check_for_pause)
@@ -747,7 +852,11 @@ class FileDownloader:
             # we're down to 1*segment_size right now, but write_segment()
             # will decrypt a copy of the segment internally, which will push
             # us up to 2*segment_size while it runs.
+            started_decrypt = time.time()
             self._output.write_segment(segment)
+            if self._results:
+                elapsed = time.time() - started_decrypt
+                self._results.timings["cumulative_decrypt"] += elapsed
         d.addCallback(_done)
         return d
 
@@ -755,12 +864,31 @@ class FileDownloader:
         self.log("downloading seg#%d of %d (%d%%)"
                  % (segnum, self._total_segments,
                     100.0 * segnum / self._total_segments))
-        segmentdler = SegmentDownloader(self, segnum, self._num_needed_shares)
+        segmentdler = SegmentDownloader(self, segnum, self._num_needed_shares,
+                                        self._results)
+        started = time.time()
         d = segmentdler.start()
+        def _finished_fetching(res):
+            elapsed = time.time() - started
+            self._results.timings["cumulative_fetch"] += elapsed
+            return res
+        if self._results:
+            d.addCallback(_finished_fetching)
         # pause before using more memory
         d.addCallback(self._check_for_pause)
+        def _started_decode(res):
+            self._started_decode = time.time()
+            return res
+        if self._results:
+            d.addCallback(_started_decode)
         d.addCallback(lambda (shares, shareids):
                       self._tail_codec.decode(shares, shareids))
+        def _finished_decode(res):
+            elapsed = time.time() - self._started_decode
+            self._results.timings["cumulative_decode"] += elapsed
+            return res
+        if self._results:
+            d.addCallback(_finished_decode)
         # pause/check-for-stop just before writing, to honor stopProducing
         d.addCallback(self._check_for_pause)
         def _done(buffers):
@@ -772,12 +900,20 @@ class FileDownloader:
             pad_size = mathutil.pad_size(self._size, self._segment_size)
             tail_size = self._segment_size - pad_size
             segment = segment[:tail_size]
+            started_decrypt = time.time()
             self._output.write_segment(segment)
+            if self._results:
+                elapsed = time.time() - started_decrypt
+                self._results.timings["cumulative_decrypt"] += elapsed
         d.addCallback(_done)
         return d
 
     def _done(self, res):
         self.log("download done")
+        if self._results:
+            now = time.time()
+            self._results.timings["total"] = now - self._started
+            self._results.timings["segments"] = now - self._started_fetching
         self._output.close()
         if self.check_crypttext_hash:
             _assert(self._crypttext_hash == self._output.crypttext_hash,
@@ -798,8 +934,6 @@ class FileDownloader:
 
 
 class LiteralDownloader:
-    implements(IDownloadStatus)
-
     def __init__(self, client, u, downloadable):
         self._uri = IFileURI(u)
         assert isinstance(self._uri, uri.LiteralFileURI)
@@ -808,6 +942,7 @@ class LiteralDownloader:
         s.set_storage_index(None)
         s.set_helper(False)
         s.set_status("Done")
+        s.set_active(False)
         s.set_progress(1.0)
 
     def start(self):
@@ -889,10 +1024,12 @@ class Downloader(service.MultiService):
     """
     implements(IDownloader)
     name = "downloader"
+    MAX_DOWNLOAD_STATUSES = 10
 
     def __init__(self):
         service.MultiService.__init__(self)
         self._all_downloads = weakref.WeakKeyDictionary()
+        self._recent_download_status = []
 
     def download(self, u, t):
         assert self.parent
@@ -907,7 +1044,10 @@ class Downloader(service.MultiService):
             dl = FileDownloader(self.parent, u, t)
         else:
             raise RuntimeError("I don't know how to download a %s" % u)
-        self._all_downloads[dl.get_download_status()] = None
+        self._all_downloads[dl] = None
+        self._recent_download_status.append(dl.get_download_status())
+        while len(self._recent_download_status) > self.MAX_DOWNLOAD_STATUSES:
+            self._recent_download_status.pop(0)
         d = dl.start()
         return d
 
@@ -920,5 +1060,10 @@ class Downloader(service.MultiService):
         return self.download(uri, FileHandle(filehandle))
 
 
-    def list_downloads(self):
+    def list_all_downloads(self):
         return self._all_downloads.keys()
+    def list_active_downloads(self):
+        return [d.get_download_status() for d in self._all_downloads.keys()
+                if d.get_download_status().get_active()]
+    def list_recent_downloads(self):
+        return self._recent_download_status
