@@ -1,15 +1,16 @@
 # -*- test-case-name: allmydata.test.test_encode -*-
 
+import time
 from zope.interface import implements
 from twisted.internet import defer
 from foolscap import eventual
-from allmydata import uri
+from allmydata import storage, uri
 from allmydata.hashtree import HashTree
-from allmydata.util import mathutil, hashutil, idlib
-from allmydata.util.assertutil import _assert
+from allmydata.util import mathutil, hashutil, base32, log
+from allmydata.util.assertutil import _assert, precondition
 from allmydata.codec import CRSEncoder
 from allmydata.interfaces import IEncoder, IStorageBucketWriter, \
-     IEncryptedUploadable
+     IEncryptedUploadable, IUploadStatus
 
 """
 The goal of the encoder is to turn the original file into a series of
@@ -36,9 +37,10 @@ information necessary to validate the data upon retrieval. Only one segment
 is handled at a time: all blocks for segment A are delivered before any
 work is begun on segment B.
 
-As blocks are created, we retain the hash of each one. The list of
-block hashes for a single share (say, hash(A1), hash(B1), hash(C1)) is
-used to form the base of a Merkle hash tree for that share (hashtrees[1]).
+As blocks are created, we retain the hash of each one. The list of block hashes
+for a single share (say, hash(A1), hash(B1), hash(C1)) is used to form the base
+of a Merkle hash tree for that share, called the block hash tree.
+
 This hash tree has one terminal leaf per block. The complete block hash
 tree is sent to the shareholder after all the data has been sent. At
 retrieval time, the decoder will ask for specific pieces of this tree before
@@ -61,6 +63,9 @@ hash tree is put into the URI.
 class NotEnoughPeersError(Exception):
     pass
 
+class UploadAborted(Exception):
+    pass
+
 KiB=1024
 MiB=1024*KiB
 GiB=1024*MiB
@@ -69,65 +74,60 @@ PiB=1024*TiB
 
 class Encoder(object):
     implements(IEncoder)
-    NEEDED_SHARES = 3
-    SHARES_OF_HAPPINESS = 7
-    TOTAL_SHARES = 10
-    MAX_SEGMENT_SIZE = 1*MiB
 
-    def __init__(self, options={}, parent=None):
+    def __init__(self, log_parent=None, upload_status=None):
         object.__init__(self)
-        self.MAX_SEGMENT_SIZE = options.get("max_segment_size",
-                                            self.MAX_SEGMENT_SIZE)
-        k,happy,n = options.get("needed_and_happy_and_total_shares",
-                                (self.NEEDED_SHARES,
-                                 self.SHARES_OF_HAPPINESS,
-                                 self.TOTAL_SHARES))
-        self.NEEDED_SHARES = k
-        self.SHARES_OF_HAPPINESS = happy
-        self.TOTAL_SHARES = n
         self.uri_extension_data = {}
         self._codec = None
-        self._parent = parent
-        if self._parent:
-            self._log_number = self._parent.log("starting Encoder %s" % self)
+        self._status = None
+        if upload_status:
+            self._status = IUploadStatus(upload_status)
+        precondition(log_parent is None or isinstance(log_parent, int),
+                     log_parent)
+        self._log_number = log.msg("creating Encoder %s" % self,
+                                   facility="tahoe.encoder", parent=log_parent)
+        self._aborted = False
 
     def __repr__(self):
         if hasattr(self, "_storage_index"):
-            return "<Encoder for %s>" % idlib.b2a(self._storage_index)[:6]
+            return "<Encoder for %s>" % storage.si_b2a(self._storage_index)[:5]
         return "<Encoder for unknown storage index>"
 
-    def log(self, msg, parent=None):
-        if not self._parent:
-            return
-        if parent is None:
-            parent = self._log_number
-        return self._parent.log(msg, parent=parent)
+    def log(self, *args, **kwargs):
+        if "parent" not in kwargs:
+            kwargs["parent"] = self._log_number
+        if "facility" not in kwargs:
+            kwargs["facility"] = "tahoe.encoder"
+        return log.msg(*args, **kwargs)
 
-    def set_size(self, size):
+    def set_encrypted_uploadable(self, uploadable):
+        eu = self._uploadable = IEncryptedUploadable(uploadable)
+        d = eu.get_size()
+        def _got_size(size):
+            self.log(format="file size: %(size)d", size=size)
+            self.file_size = size
+        d.addCallback(_got_size)
+        d.addCallback(lambda res: eu.get_all_encoding_parameters())
+        d.addCallback(self._got_all_encoding_parameters)
+        d.addCallback(lambda res: eu.get_storage_index())
+        def _done(storage_index):
+            self._storage_index = storage_index
+            return self
+        d.addCallback(_done)
+        return d
+
+    def _got_all_encoding_parameters(self, params):
         assert not self._codec
-        self.file_size = size
-
-    def set_params(self, encoding_parameters):
-        assert not self._codec
-        k,d,n = encoding_parameters
-        self.NEEDED_SHARES = k
-        self.SHARES_OF_HAPPINESS = d
-        self.TOTAL_SHARES = n
-        self.log("set_params: %d,%d,%d" % (k, d, n))
-
-    def _setup_codec(self):
-        self.num_shares = self.TOTAL_SHARES
-        self.required_shares = self.NEEDED_SHARES
-        self.shares_of_happiness = self.SHARES_OF_HAPPINESS
-
-        self.segment_size = min(self.MAX_SEGMENT_SIZE, self.file_size)
-        # this must be a multiple of self.required_shares
-        self.segment_size = mathutil.next_multiple(self.segment_size,
-                                                   self.required_shares)
-
-        # now set up the codec
+        k, happy, n, segsize = params
+        self.required_shares = k
+        self.shares_of_happiness = happy
+        self.num_shares = n
+        self.segment_size = segsize
+        self.log("got encoding parameters: %d/%d/%d %d" % (k,happy,n, segsize))
+        self.log("now setting up codec")
 
         assert self.segment_size % self.required_shares == 0
+
         self.num_segments = mathutil.div_ceil(self.file_size,
                                               self.segment_size)
 
@@ -172,22 +172,8 @@ class Encoder(object):
     def _compute_overhead(self):
         return 0
 
-    def set_encrypted_uploadable(self, uploadable):
-        u = self._uploadable = IEncryptedUploadable(uploadable)
-        d = u.get_size()
-        d.addCallback(self.set_size)
-        d.addCallback(lambda res: self.get_param("serialized_params"))
-        d.addCallback(u.set_serialized_encoding_parameters)
-        d.addCallback(lambda res: u.get_storage_index())
-        def _done(storage_index):
-            self._storage_index = storage_index
-            return self
-        d.addCallback(_done)
-        return d
-
     def get_param(self, name):
-        if not self._codec:
-            self._setup_codec()
+        assert self._codec
 
         if name == "storage_index":
             return self._storage_index
@@ -214,11 +200,9 @@ class Encoder(object):
         self.landlords = landlords.copy()
 
     def start(self):
-        self.log("starting")
+        self.log("%s starting" % (self,))
         #paddedsize = self._size + mathutil.pad_size(self._size, self.needed_shares)
-        if not self._codec:
-            self._setup_codec()
-
+        assert self._codec
         self._crypttext_hasher = hashutil.crypttext_hasher()
         self._crypttext_hashes = []
         self.segment_num = 0
@@ -228,12 +212,17 @@ class Encoder(object):
         # that we sent to that landlord.
         self.share_root_hashes = [None] * self.num_shares
 
-        d = eventual.fireEventually()
-        d.addCallback(lambda res:
-                      self._uploadable.set_segment_size(self.segment_size))
+        self._times = {
+            "cumulative_encoding": 0.0,
+            "cumulative_sending": 0.0,
+            "hashes_and_close": 0.0,
+            "total_encode_and_push": 0.0,
+            }
+        self._start_total_timestamp = time.time()
 
-        for l in self.landlords.values():
-            d.addCallback(lambda res, l=l: l.start())
+        d = eventual.fireEventually()
+
+        d.addCallback(lambda res: self.start_all_shareholders())
 
         for i in range(self.num_segments-1):
             # note to self: this form doesn't work, because lambda only
@@ -262,6 +251,27 @@ class Encoder(object):
         d.addCallbacks(lambda res: self.done(), self.err)
         return d
 
+    def set_status(self, status):
+        if self._status:
+            self._status.set_status(status)
+
+    def set_encode_and_push_progress(self, sent_segments=None, extra=0.0):
+        if self._status:
+            # we treat the final hash+close as an extra segment
+            if sent_segments is None:
+                sent_segments = self.num_segments
+            progress = float(sent_segments + extra) / (self.num_segments + 1)
+            self._status.set_progress(2, progress)
+
+    def abort(self):
+        self.log("aborting upload", level=log.UNUSUAL)
+        assert self._codec, "don't call abort before start"
+        self._aborted = True
+        # the next segment read (in _gather_data inside _encode_segment) will
+        # raise UploadAborted(), which will bypass the rest of the upload
+        # chain. If we've sent the final segment's shares, it's too late to
+        # abort. TODO: allow abort any time up to close_all_shareholders.
+
     def _turn_barrier(self, res):
         # putting this method in a Deferred chain imposes a guaranteed
         # reactor turn between the pre- and post- portions of that chain.
@@ -272,8 +282,20 @@ class Encoder(object):
 
         return eventual.fireEventually(res)
 
+
+    def start_all_shareholders(self):
+        self.log("starting shareholders", level=log.NOISY)
+        self.set_status("Starting shareholders")
+        dl = []
+        for shareid in self.landlords:
+            d = self.landlords[shareid].start()
+            d.addErrback(self._remove_shareholder, shareid, "start")
+            dl.append(d)
+        return self._gather_responses(dl)
+
     def _encode_segment(self, segnum):
         codec = self._codec
+        start = time.time()
 
         # the ICodecEncoder API wants to receive a total of self.segment_size
         # bytes on each encode() call, broken up into a number of
@@ -302,17 +324,23 @@ class Encoder(object):
 
         d = self._gather_data(self.required_shares, input_piece_size,
                               crypttext_segment_hasher)
-        def _done(chunks):
+        def _done_gathering(chunks):
             for c in chunks:
                 assert len(c) == input_piece_size
             self._crypttext_hashes.append(crypttext_segment_hasher.digest())
             # during this call, we hit 5*segsize memory
             return codec.encode(chunks)
+        d.addCallback(_done_gathering)
+        def _done(res):
+            elapsed = time.time() - start
+            self._times["cumulative_encoding"] += elapsed
+            return res
         d.addCallback(_done)
         return d
 
     def _encode_tail_segment(self, segnum):
 
+        start = time.time()
         codec = self._tail_codec
         input_piece_size = codec.get_block_size()
 
@@ -321,13 +349,18 @@ class Encoder(object):
         d = self._gather_data(self.required_shares, input_piece_size,
                               crypttext_segment_hasher,
                               allow_short=True)
-        def _done(chunks):
+        def _done_gathering(chunks):
             for c in chunks:
                 # a short trailing chunk will have been padded by
                 # _gather_data
                 assert len(c) == input_piece_size
             self._crypttext_hashes.append(crypttext_segment_hasher.digest())
             return codec.encode(chunks)
+        d.addCallback(_done_gathering)
+        def _done(res):
+            elapsed = time.time() - start
+            self._times["cumulative_encoding"] += elapsed
+            return res
         d.addCallback(_done)
         return d
 
@@ -340,11 +373,16 @@ class Encoder(object):
         with the combination of any 'previous_chunks' and the new chunks
         which were gathered."""
 
+        if self._aborted:
+            raise UploadAborted()
+
         if not num_chunks:
             return defer.succeed(previous_chunks)
 
-        d = self._uploadable.read_encrypted(input_chunk_size)
+        d = self._uploadable.read_encrypted(input_chunk_size, False)
         def _got(data):
+            if self._aborted:
+                raise UploadAborted()
             encrypted_pieces = []
             length = 0
             while data:
@@ -361,7 +399,12 @@ class Encoder(object):
                     encrypted_pieces.append('\x00' * pad_size)
             else:
                 # non-tail segments should be the full segment size
-                assert length == input_chunk_size
+                if length != input_chunk_size:
+                    log.msg("non-tail segment should be full segment size: %d!=%d"
+                            % (length, input_chunk_size), level=log.BAD)
+                precondition(length == input_chunk_size,
+                             "length=%d != input_chunk_size=%d" %
+                             (length, input_chunk_size))
 
             encrypted_piece = "".join(encrypted_pieces)
             return previous_chunks + [encrypted_piece]
@@ -381,19 +424,23 @@ class Encoder(object):
         # *doesn't* have a share, that's an error.
         _assert(set(self.landlords.keys()).issubset(set(shareids)),
                 shareids=shareids, landlords=self.landlords)
+        start = time.time()
         dl = []
-        lognum = self.log("send_segment(%d)" % segnum)
+        self.set_status("Sending segment %d of %d" % (segnum+1,
+                                                      self.num_segments))
+        self.set_encode_and_push_progress(segnum)
+        lognum = self.log("send_segment(%d)" % segnum, level=log.NOISY)
         for i in range(len(shares)):
             subshare = shares[i]
             shareid = shareids[i]
             d = self.send_subshare(shareid, segnum, subshare, lognum)
             dl.append(d)
             subshare_hash = hashutil.block_hash(subshare)
-            #from allmydata.util import idlib
+            #from allmydata.util import base32
             #log.msg("creating block (shareid=%d, blocknum=%d) "
             #        "len=%d %r .. %r: %s" %
             #        (shareid, segnum, len(subshare),
-            #         subshare[:50], subshare[-50:], idlib.b2a(subshare_hash)))
+            #         subshare[:50], subshare[-50:], base32.b2a(subshare_hash)))
             self.subshare_hashes[shareid].append(subshare_hash)
 
         dl = self._gather_responses(dl)
@@ -403,7 +450,10 @@ class Encoder(object):
                       self.segment_size*(segnum+1),
                       self.segment_size*self.num_segments,
                       100 * (segnum+1) / self.num_segments,
-                      ))
+                      ),
+                     level=log.OPERATIONAL)
+            elapsed = time.time() - start
+            self._times["cumulative_sending"] += elapsed
             return res
         dl.addCallback(_logit)
         return dl
@@ -413,10 +463,10 @@ class Encoder(object):
             return defer.succeed(None)
         sh = self.landlords[shareid]
         lognum2 = self.log("put_block to %s" % self.landlords[shareid],
-                           parent=lognum)
+                           parent=lognum, level=log.NOISY)
         d = sh.put_block(segment_num, subshare)
         def _done(res):
-            self.log("put_block done", parent=lognum2)
+            self.log("put_block done", parent=lognum2, level=log.NOISY)
             return res
         d.addCallback(_done)
         d.addErrback(self._remove_shareholder, shareid,
@@ -424,13 +474,15 @@ class Encoder(object):
         return d
 
     def _remove_shareholder(self, why, shareid, where):
-        ln = self.log("UNUSUAL: error while sending %s to shareholder=%d: %s" %
-                      (where, shareid, why))
+        ln = self.log(format="error while sending %(method)s to shareholder=%(shnum)d",
+                      method=where, shnum=shareid,
+                      level=log.UNUSUAL, failure=why)
         if shareid in self.landlords:
             del self.landlords[shareid]
         else:
             # even more UNUSUAL
-            self.log("WEIRD: they weren't in our list of landlords", parent=ln)
+            self.log("they weren't in our list of landlords", parent=ln,
+                     level=log.WEIRD)
         if len(self.landlords) < self.shares_of_happiness:
             msg = "lost too many shareholders during upload: %s" % why
             raise NotEnoughPeersError(msg)
@@ -455,22 +507,32 @@ class Encoder(object):
         return d
 
     def finish_hashing(self):
+        self._start_hashing_and_close_timestamp = time.time()
+        self.set_status("Finishing hashes")
+        self.set_encode_and_push_progress(extra=0.0)
         crypttext_hash = self._crypttext_hasher.digest()
         self.uri_extension_data["crypttext_hash"] = crypttext_hash
-        u = self._uploadable
-        d = u.get_plaintext_hash()
+        d = self._uploadable.get_plaintext_hash()
         def _got(plaintext_hash):
             self.uri_extension_data["plaintext_hash"] = plaintext_hash
-            return u.get_plaintext_segment_hashtree_nodes(self.num_segments)
+            return self._uploadable.get_plaintext_hashtree_leaves(0, self.num_segments, self.num_segments)
         d.addCallback(_got)
-        def _got_hashtree_nodes(t):
-            self.uri_extension_data["plaintext_root_hash"] = t[0]
-            self._plaintext_hashtree_nodes = t
-        d.addCallback(_got_hashtree_nodes)
+        def _got_hashtree_leaves(leaves):
+            self.log("Encoder: got plaintext_hashtree_leaves: %s" %
+                     (",".join([base32.b2a(h) for h in leaves]),),
+                     level=log.NOISY)
+            ht = list(HashTree(list(leaves)))
+            self.uri_extension_data["plaintext_root_hash"] = ht[0]
+            self._plaintext_hashtree_nodes = ht
+        d.addCallback(_got_hashtree_leaves)
+
+        d.addCallback(lambda res: self._uploadable.close())
         return d
 
     def send_plaintext_hash_tree_to_all_shareholders(self):
-        self.log("sending plaintext hash tree")
+        self.log("sending plaintext hash tree", level=log.NOISY)
+        self.set_status("Sending Plaintext Hash Tree")
+        self.set_encode_and_push_progress(extra=0.2)
         dl = []
         for shareid in self.landlords.keys():
             d = self.send_plaintext_hash_tree(shareid,
@@ -487,7 +549,9 @@ class Encoder(object):
         return d
 
     def send_crypttext_hash_tree_to_all_shareholders(self):
-        self.log("sending crypttext hash tree")
+        self.log("sending crypttext hash tree", level=log.NOISY)
+        self.set_status("Sending Crypttext Hash Tree")
+        self.set_encode_and_push_progress(extra=0.3)
         t = HashTree(self._crypttext_hashes)
         all_hashes = list(t)
         self.uri_extension_data["crypttext_root_hash"] = t[0]
@@ -505,7 +569,9 @@ class Encoder(object):
         return d
 
     def send_all_subshare_hash_trees(self):
-        self.log("sending subshare hash trees")
+        self.log("sending subshare hash trees", level=log.NOISY)
+        self.set_status("Sending Subshare Hash Trees")
+        self.set_encode_and_push_progress(extra=0.4)
         dl = []
         for shareid,hashes in enumerate(self.subshare_hashes):
             # hashes is a list of the hashes of all subshares that were sent
@@ -532,7 +598,9 @@ class Encoder(object):
         # validate their share. This includes the share hash itself, but does
         # not include the top-level hash root (which is stored securely in
         # the URI instead).
-        self.log("sending all share hash trees")
+        self.log("sending all share hash trees", level=log.NOISY)
+        self.set_status("Sending Share Hash Trees")
+        self.set_encode_and_push_progress(extra=0.6)
         dl = []
         for h in self.share_root_hashes:
             assert h
@@ -558,12 +626,21 @@ class Encoder(object):
         return d
 
     def send_uri_extension_to_all_shareholders(self):
-        self.log("sending uri_extension")
+        lp = self.log("sending uri_extension", level=log.NOISY)
+        self.set_status("Sending URI Extensions")
+        self.set_encode_and_push_progress(extra=0.8)
         for k in ('crypttext_root_hash', 'crypttext_hash',
                   'plaintext_root_hash', 'plaintext_hash',
                   ):
             assert k in self.uri_extension_data
         uri_extension = uri.pack_extension(self.uri_extension_data)
+        ed = {}
+        for k,v in self.uri_extension_data.items():
+            if k.endswith("hash"):
+                ed[k] = base32.b2a(v)
+            else:
+                ed[k] = v
+        self.log("uri_extension_data is %s" % (ed,), level=log.NOISY, parent=lp)
         self.uri_extension_hash = hashutil.uri_extension_hash(uri_extension)
         dl = []
         for shareid in self.landlords.keys():
@@ -577,7 +654,9 @@ class Encoder(object):
         return d
 
     def close_all_shareholders(self):
-        self.log("closing shareholders")
+        self.log("closing shareholders", level=log.NOISY)
+        self.set_status("Closing Shareholders")
+        self.set_encode_and_push_progress(extra=0.9)
         dl = []
         for shareid in self.landlords:
             d = self.landlords[shareid].close()
@@ -586,12 +665,47 @@ class Encoder(object):
         return self._gather_responses(dl)
 
     def done(self):
-        self.log("upload done")
+        self.log("upload done", level=log.OPERATIONAL)
+        self.set_status("Done")
+        self.set_encode_and_push_progress(extra=1.0) # done
+        now = time.time()
+        h_and_c_elapsed = now - self._start_hashing_and_close_timestamp
+        self._times["hashes_and_close"] = h_and_c_elapsed
+        total_elapsed = now - self._start_total_timestamp
+        self._times["total_encode_and_push"] = total_elapsed
+
+        # update our sharemap
+        self._shares_placed = set(self.landlords.keys())
         return (self.uri_extension_hash, self.required_shares,
                 self.num_shares, self.file_size)
 
     def err(self, f):
-        self.log("UNUSUAL: %s: upload failed: %s" % (self, f))
-        if f.check(defer.FirstError):
-            return f.value.subFailure
-        return f
+        self.log("upload failed", failure=f, level=log.UNUSUAL)
+        self.set_status("Failed")
+        # we need to abort any remaining shareholders, so they'll delete the
+        # partial share, allowing someone else to upload it again.
+        self.log("aborting shareholders", level=log.UNUSUAL)
+        dl = []
+        for shareid in list(self.landlords.keys()):
+            d = self.landlords[shareid].abort()
+            d.addErrback(self._remove_shareholder, shareid, "abort")
+            dl.append(d)
+        d = self._gather_responses(dl)
+        def _done(res):
+            self.log("shareholders aborted", level=log.UNUSUAL)
+            if f.check(defer.FirstError):
+                return f.value.subFailure
+            return f
+        d.addCallback(_done)
+        return d
+
+    def get_shares_placed(self):
+        # return a set of share numbers that were successfully placed.
+        return self._shares_placed
+
+    def get_times(self):
+        # return a dictionary of encode+push timings
+        return self._times
+
+    def get_uri_extension_data(self):
+        return self.uri_extension_data
