@@ -5,10 +5,11 @@ BACKEND = 'native_client.php'
 ACCOUNT_PAGE = 'account'
 WELCOME_PAGE = 'welcome_install'
 TAHOESVC_NAME = 'Tahoe'
-WINFUSESVC_NAME = 'Allmydata Tahoe SMB'
+WINFUSESVC_NAME = 'Allmydata SMB'
 
 import os
 import re
+import socket
 import sys
 #import time
 import traceback
@@ -63,12 +64,13 @@ def create_account(url, user, passwd, subscribe):
     result_code = post(url, args)
     return result_code
 
-def record_install(url, user, passwd, nodeid):
+def record_install(url, user, passwd, nodeid, nickname):
     args = {
         'action': 'record_install',
         'email': unicode_to_utf8(user),
         'passwd': unicode_to_utf8(passwd),
         'nodeid': nodeid,
+        'moniker': nickname,
         }
     result_code = post(url, args)
     return result_code
@@ -116,6 +118,31 @@ def get_nodeid():
     tub = foolscap.Tub(certFile=certfile)
     return tub.getTubID()
 
+def get_nickname():
+    nick = None
+    nnfile = os.path.join(get_basedir(), 'nickname')
+    if os.path.exists(nnfile):
+        try:
+            fh = file(nnfile, 'rb')
+            nick = fh.read().strip()
+            fh.close()
+        except:
+            DisplayTraceback('Failed to read existing nickname file %s' % (nnfile,))
+    if not nick:
+        nick = socket.gethostname()
+    return nick
+
+def maybe_write_file(filename, contents):
+    fname = os.path.join(get_basedir(), filename)
+    try:
+        if not os.path.exists(fname):
+            fh = file(fname, 'wb')
+            fh.write(contents)
+            fh.write('\n')
+            fh.close()
+    except:
+        DisplayTraceback('Failed to write file %s' % (fname,))
+
 def configure(backend, user, passwd):
     _config_re = re.compile('^([^:]*): (.*)$')
     config = get_config(backend, user, passwd)
@@ -190,7 +217,7 @@ class ConfWizApp(wx.App):
 
 class LoginFrame(wx.Frame):
     def __init__(self, app):
-        title = 'Allmydata Tahoe Config Wizard'
+        title = 'Allmydata Config Wizard'
         wx.Frame.__init__(self, None, -1, title)
         self.app = app
         self.SetIcon(amdicon.getIcon())
@@ -220,7 +247,7 @@ class LoginFrame(wx.Frame):
 
 class RegisterFrame(wx.Frame):
     def __init__(self, app):
-        title = 'Allmydata Tahoe Config Wizard'
+        title = 'Allmydata Config Wizard'
         wx.Frame.__init__(self, None, -1, title)
         self.app = app
         self.SetIcon(amdicon.getIcon())
@@ -314,12 +341,15 @@ class LoginPanel(wx.Panel):
             return
 
         nodeid = get_nodeid()
-        ret = record_install(backend, user, passwd, nodeid)
+        nickname = get_nickname()
+        ret = record_install(backend, user, passwd, nodeid, nickname)
         if ret != 'ok':
             wx.MessageBox('Error "%s" recording this system (%s)' % (ret, nodeid), 'Error')
 
         configure(backend, user, passwd)
         maybe_start_services()
+        maybe_write_file('nickname', nickname)
+        maybe_write_file('accountname', user)
 
         self.app.open_welcome_page()
 
@@ -459,12 +489,15 @@ class RegisterPanel(wx.Panel):
             return
 
         nodeid = get_nodeid()
-        ret = record_install(backend, user, passwd, nodeid)
+        nickname = get_nickname()
+        ret = record_install(backend, user, passwd, nodeid, nickname)
         if ret != 'ok':
             wx.MessageBox('Error "%s" recording this system (%s)' % (ret, nodeid), 'Error')
 
         configure(backend, user, passwd)
         maybe_start_services()
+        maybe_write_file('nickname', nickname)
+        maybe_write_file('accountname', user)
 
         self.app.open_welcome_page()
 
