@@ -1,7 +1,7 @@
 
 from zope.interface import Interface
 from foolscap.schema import StringConstraint, ListOf, TupleOf, SetOf, DictOf, \
-     ChoiceOf
+     ChoiceOf, IntegerConstraint
 from foolscap import RemoteInterface, Referenceable
 
 HASH_SIZE=32
@@ -19,6 +19,9 @@ MAX_BUCKETS = 200  # per peer
 # MAX_SEGMENT_SIZE in encode.py is 1 MiB (this constraint allows k = 1)
 ShareData = StringConstraint(2**20)
 URIExtensionData = StringConstraint(1000)
+Number = IntegerConstraint(8) # 2**(8*8) == 16EiB ~= 18e18 ~= 18 exabytes
+Offset = Number
+ReadSize = int # the 'int' constraint is 2**31 == 2Gib
 LeaseRenewSecret = Hash # used to protect bucket lease renewal requests
 LeaseCancelSecret = Hash # used to protect bucket lease cancellation requests
 
@@ -157,9 +160,16 @@ class IIntroducerClient(Interface):
         """Returns a boolean, True if we are currently connected to the
         introducer, False if not."""
 
+class RIStubClient(RemoteInterface):
+    """Each client publishes a service announcement for a dummy object called
+    the StubClient. This object doesn't actually offer any services, but the
+    announcement helps the Introducer keep track of which clients are
+    subscribed (so the grid admin can keep track of things like the size of
+    the grid and the client versions in use. This is the (empty)
+    RemoteInterface for the StubClient."""
 
 class RIBucketWriter(RemoteInterface):
-    def write(offset=int, data=ShareData):
+    def write(offset=Offset, data=ShareData):
         return None
 
     def close():
@@ -176,21 +186,24 @@ class RIBucketWriter(RemoteInterface):
         return None
 
 class RIBucketReader(RemoteInterface):
-    def read(offset=int, length=int):
+    def read(offset=Offset, length=ReadSize):
+        # ShareData is limited to 1MiB, so we don't need length= to be any
+        # larger than that. Large files must be read in pieces.
         return ShareData
 
-TestVector = ListOf(TupleOf(int, int, str, str))
+TestVector = ListOf(TupleOf(Offset, ReadSize, str, str))
 # elements are (offset, length, operator, specimen)
 # operator is one of "lt, le, eq, ne, ge, gt"
 # nop always passes and is used to fetch data while writing.
 # you should use length==len(specimen) for everything except nop
-DataVector = ListOf(TupleOf(int, ShareData))
+DataVector = ListOf(TupleOf(Offset, ShareData))
 # (offset, data). This limits us to 30 writes of 1MiB each per call
 TestAndWriteVectorsForShares = DictOf(int,
                                       TupleOf(TestVector,
                                               DataVector,
-                                              ChoiceOf(None, int))) # new_length
-ReadVector = ListOf(TupleOf(int, int))
+                                              ChoiceOf(None, Offset), # new_length
+                                              ))
+ReadVector = ListOf(TupleOf(Offset, ReadSize))
 ReadData = ListOf(ShareData)
 # returns data[offset:offset+length] for each element of TestVector
 
@@ -214,7 +227,7 @@ class RIStorageServer(RemoteInterface):
                          renew_secret=LeaseRenewSecret,
                          cancel_secret=LeaseCancelSecret,
                          sharenums=SetOf(int, maxLength=MAX_BUCKETS),
-                         allocated_size=int, canary=Referenceable):
+                         allocated_size=Offset, canary=Referenceable):
         """
         @param storage_index: the index of the bucket to be created or
                               increfed.
@@ -563,17 +576,45 @@ class IMutableFileNode(IFileNode, IMutableFilesystemNode):
         more advanced API will signal and provide access to the multiple
         heads."""
 
-    def replace(newdata):
-        """Replace the old contents with the new data. Returns a Deferred
-        that fires (with None) when the operation is complete.
+    def update(newdata):
+        """Attempt to replace the old contents with the new data.
 
-        If the node detects that there are multiple outstanding versions of
-        the file, this will raise ConsistencyError, and may leave the
-        distributed file in an unusual state (the node will try to ensure
-        that at least one version of the file remains retrievable, but it may
-        or may not be the one you just tried to upload). You should respond
-        to this by downloading the current contents of the file and retrying
-        the replace() operation.
+        download_to_data() must have been called before calling update().
+
+        Returns a Deferred. If the Deferred fires successfully, the update
+        appeared to succeed. However, another writer (who read before your
+        changes were published) might still clobber your changes: they will
+        discover a problem but you will not. (see ticket #347 for details).
+
+        If the mutable file has been changed (by some other writer) since the
+        last call to download_to_data(), this will raise
+        UncoordinatedWriteError and the file will be left in an inconsistent
+        state (possibly the version you provided, possibly the old version,
+        possibly somebody else's version, and possibly a mix of shares from
+        all of these). The recommended response to UncoordinatedWriteError is
+        to either return it to the caller (since they failed to coordinate
+        their writes), or to do a new download_to_data() / modify-data /
+        update() loop.
+
+        update() is appropriate to use in a read-modify-write sequence, such
+        as a directory modification.
+        """
+
+    def overwrite(newdata):
+        """Attempt to replace the old contents with the new data.
+
+        Unlike update(), overwrite() does not require a previous call to
+        download_to_data(). It will unconditionally replace the old contents
+        with new data.
+
+        overwrite() is implemented by doing download_to_data() and update()
+        in rapid succession, so there remains a (smaller) possibility of
+        UncoordinatedWriteError. A future version will remove the full
+        download_to_data step, making this faster than update().
+
+        overwrite() is only appropriate to use when the new contents of the
+        mutable file are completely unrelated to the old ones, and you do not
+        care about other clients changes to the file.
         """
 
     def get_writekey():
@@ -669,7 +710,7 @@ class IDirectoryNode(IMutableFilesystemNode):
         If this directory node is read-only, the Deferred will errback with a
         NotMutableError."""
 
-    def set_uris(entries):
+    def set_children(entries):
         """Add multiple (name, child_uri) pairs (or (name, child_uri,
         metadata) triples) to a directory node. Returns a Deferred that fires
         (with None) when the operation finishes. This is equivalent to
@@ -1278,9 +1319,53 @@ class IUploadable(Interface):
 
 class IUploadResults(Interface):
     """I am returned by upload() methods. I contain a number of public
-    attributes which can be read to determine the results of the upload::
+    attributes which can be read to determine the results of the upload. Some
+    of these are functional, some are timing information. All of these may be
+    None.::
 
+     .file_size : the size of the file, in bytes
      .uri : the CHK read-cap for the file
+     .ciphertext_fetched : how many bytes were fetched by the helper
+     .sharemap : dict mapping share number to placement string
+     .servermap : dict mapping server peerid to a set of share numbers
+     .timings : dict of timing information, mapping name to seconds (float)
+       total : total upload time, start to finish
+       storage_index : time to compute the storage index
+       peer_selection : time to decide which peers will be used
+       contacting_helper : initial helper query to upload/no-upload decision
+       existence_check : helper pre-upload existence check
+       helper_total : initial helper query to helper finished pushing
+       cumulative_fetch : helper waiting for ciphertext requests
+       total_fetch : helper start to last ciphertext response
+       cumulative_encoding : just time spent in zfec
+       cumulative_sending : just time spent waiting for storage servers
+       hashes_and_close : last segment push to shareholder close
+       total_encode_and_push : first encode to shareholder close
+
+    """
+
+class IDownloadResults(Interface):
+    """I am created internally by download() methods. I contain a number of
+    public attributes which contain details about the download process.::
+
+     .file_size : the size of the file, in bytes
+     .servers_used : set of server peerids that were used during download
+     .server_problems : dict mapping server peerid to a problem string. Only
+                        servers that had problems (bad hashes, disconnects) are
+                        listed here.
+     .servermap : dict mapping server peerid to a set of share numbers. Only
+                  servers that had any shares are listed here.
+     .timings : dict of timing information, mapping name to seconds (float)
+       peer_selection : time to ask servers about shares
+       servers_peer_selection : dict of peerid to DYHB-query time
+       uri_extension : time to fetch a copy of the URI extension block
+       hashtrees : time to fetch the hash trees
+       segments : time to fetch, decode, and deliver segments
+       cumulative_fetch : time spent waiting for storage servers
+       cumulative_decode : just time spent in zfec
+       cumulative_decrypt : just time spent in decryption
+       total : total download time, start to finish
+       fetch_per_server : dict of peerid to list of per-segment fetch times
 
     """
 
@@ -1388,14 +1473,30 @@ class IClient(Interface):
         """
 
 class IClientStatus(Interface):
-    def list_uploads():
-        """Return a list of IUploadStatus objects, one for each
-        upload which is currently running."""
-    def list_downloads():
-        """Return a list of IDownloadStatus objects, one for each
-        download which is currently running."""
+    def list_all_uploads():
+        """Return a list of uploader objects, one for each upload which
+        currently has an object available (tracked with weakrefs). This is
+        intended for debugging purposes."""
+    def list_active_uploads():
+        """Return a list of active IUploadStatus objects."""
+    def list_recent_uploads():
+        """Return a list of IUploadStatus objects for the most recently
+        started uploads."""
+
+    def list_all_downloads():
+        """Return a list of downloader objects, one for each download which
+        currently has an object available (tracked with weakrefs). This is
+        intended for debugging purposes."""
+    def list_active_downloads():
+        """Return a list of active IDownloadStatus objects."""
+    def list_recent_downloads():
+        """Return a list of IDownloadStatus objects for the most recently
+        started downloads."""
 
 class IUploadStatus(Interface):
+    def get_started():
+        """Return a timestamp (float with seconds since epoch) indicating
+        when the operation was started."""
     def get_storage_index():
         """Return a string with the (binary) storage index in use on this
         upload. Returns None if the storage index has not yet been
@@ -1420,8 +1521,21 @@ class IUploadStatus(Interface):
         process has finished: for helper uploads this is dependent upon the
         helper providing progress reports. It might be reasonable to add all
         three numbers and report the sum to the user."""
+    def get_active():
+        """Return True if the upload is currently active, False if not."""
+    def get_results():
+        """Return an instance of UploadResults (which contains timing and
+        sharemap information). Might return None if the upload is not yet
+        finished."""
+    def get_counter():
+        """Each upload status gets a unique number: this method returns that
+        number. This provides a handle to this particular upload, so a web
+        page can generate a suitable hyperlink."""
 
 class IDownloadStatus(Interface):
+    def get_started():
+        """Return a timestamp (float with seconds since epoch) indicating
+        when the operation was started."""
     def get_storage_index():
         """Return a string with the (binary) storage index in use on this
         download. This may be None if there is no storage index (i.e. LIT
@@ -1439,7 +1553,17 @@ class IDownloadStatus(Interface):
         """Returns a float (from 0.0 to 1.0) describing the amount of the
         download that has completed. This value will remain at 0.0 until the
         first byte of plaintext is pushed to the download target."""
+    def get_active():
+        """Return True if the download is currently active, False if not."""
+    def get_counter():
+        """Each download status gets a unique number: this method returns
+        that number. This provides a handle to this particular download, so a
+        web page can generate a suitable hyperlink."""
 
+class IPublishStatus(Interface):
+    pass
+class IRetrieveStatus(Interface):
+    pass
 
 class NotCapableError(Exception):
     """You have tried to write to a read-only node."""
@@ -1504,12 +1628,12 @@ class RIEncryptedUploadable(RemoteInterface):
     __remote_name__ = "RIEncryptedUploadable.tahoe.allmydata.com"
 
     def get_size():
-        return int
+        return Offset
 
     def get_all_encoding_parameters():
         return (int, int, int, long)
 
-    def read_encrypted(offset=long, length=long):
+    def read_encrypted(offset=Offset, length=ReadSize):
         return ListOf(str)
 
     def get_plaintext_hashtree_leaves(first=int, last=int, num_segments=int):
