@@ -166,16 +166,18 @@ class Node(service.MultiService):
             self._tub_ready_observerlist.fire(self)
             return self
         d.addCallback(_ready)
-        def _die(failure):
-            self.log('_startService() failed')
-            log.err(failure)
-            print "Node._startService failed, aborting"
-            print failure
-            #reactor.stop() # for unknown reasons, reactor.stop() isn't working.  [ ] TODO
-            self.log('calling os.abort()')
-            print "calling os.abort()"
-            os.abort()
-        d.addErrback(_die)
+        d.addErrback(self._service_startup_failed)
+
+    def _service_startup_failed(self, failure):
+        self.log('_startService() failed')
+        log.err(failure)
+        print "Node._startService failed, aborting"
+        print failure
+        #reactor.stop() # for unknown reasons, reactor.stop() isn't working.  [ ] TODO
+        self.log('calling os.abort()')
+        log.msg('calling os.abort()')
+        print "calling os.abort()"
+        os.abort()
 
     def stopService(self):
         self.log("Node.stopService")
