@@ -27,7 +27,11 @@ class VDriveOptions(BaseOptions, usage.Options):
     def postOptions(self):
         # compute a node-url from the existing options, put in self['node-url']
         if self['node-directory']:
-            self['node-directory'] = os.path.expanduser(self['node-directory'])
+            if sys.platform == 'win32' and self['node-directory'] == '~/.tahoe':
+                from allmydata.windows import registry
+                self['node-directory'] = registry.get_base_dir_path()
+            else:
+                self['node-directory'] = os.path.expanduser(self['node-directory'])
         if self['node-url']:
             if (not isinstance(self['node-url'], basestring)
                 or not NODEURL_RE.match(self['node-url'])):
@@ -120,6 +124,9 @@ class WebopenOptions(VDriveOptions):
 
     longdesc = """Opens a webbrowser to the contents of some portion of the virtual drive."""
 
+class ReplOptions(usage.Options):
+    pass
+
 subCommands = [
     ["ls", None, ListOptions, "List a directory"],
     ["get", None, GetOptions, "Retrieve a file from the virtual drive."],
@@ -127,6 +134,7 @@ subCommands = [
     ["rm", None, RmOptions, "Unlink a file or directory in the virtual drive."],
     ["mv", None, MvOptions, "Move a file within the virtual drive."],
     ["webopen", None, WebopenOptions, "Open a webbrowser to the root_dir"],
+    ["repl", None, ReplOptions, "Open a python interpreter"],
     ]
 
 def list(config, stdout, stderr):
@@ -209,6 +217,10 @@ def webopen(config, stdout, stderr):
     webbrowser.open(url)
     return 0
 
+def repl(config, stdout, stderr):
+    import code
+    return code.interact()
+
 dispatch = {
     "ls": list,
     "get": get,
@@ -216,5 +228,6 @@ dispatch = {
     "rm": rm,
     "mv": mv,
     "webopen": webopen,
+    "repl": repl,
     }
 
