@@ -5,7 +5,7 @@ from cStringIO import StringIO
 from twisted.internet import defer, reactor, protocol, error
 from twisted.application import service, internet
 from twisted.web import client as tw_client
-from allmydata import client, introducer
+from allmydata import client, introducer, upload
 from allmydata.scripts import create_node
 from allmydata.util import testutil, fileutil
 import foolscap
@@ -242,17 +242,15 @@ this file are ignored.
             pass
         else:
             # don't accept any shares
-            f = open(os.path.join(clientdir, "sizelimit"), "w")
-            f.write("0\n")
+            f = open(os.path.join(clientdir, "readonly_storage"), "w")
+            f.write("true\n")
             f.close()
             ## also, if we do receive any shares, throw them away
             #f = open(os.path.join(clientdir, "debug_no_storage"), "w")
             #f.write("no_storage\n")
             #f.close()
         if self.mode == "upload-self":
-            f = open(os.path.join(clientdir, "push_to_ourselves"), "w")
-            f.write("push_to_ourselves\n")
-            f.close()
+            pass
         self.keepalive_file = os.path.join(clientdir,
                                            "suicide_prevention_hotline")
         # now start updating the mtime.
@@ -393,7 +391,8 @@ this file are ignored.
             files[name] = self.create_data(name, size)
             u = self.nodes[0].getServiceNamed("uploader")
             d = self.nodes[0].debug_wait_for_client_connections(self.numnodes+1)
-            d.addCallback(lambda res: u.upload_filename(files[name]))
+            d.addCallback(lambda res: u.upload(upload.FileName(files[name])))
+            d.addCallback(lambda results: results.uri)
         else:
             raise RuntimeError("unknown mode=%s" % self.mode)
         def _complete(uri):
