@@ -7,15 +7,17 @@ from twisted.python import log
 
 import allmydata
 from allmydata import client, introducer
-from allmydata.util import idlib
+from allmydata.util import base32
 from foolscap.eventual import flushEventualQueue
 
 class FakeIntroducerClient(introducer.IntroducerClient):
     def __init__(self):
-        self.connections = {}
-
-def permute(c, key):
-    return [ y for x, y, z in c.get_permuted_peers(key) ]
+        self._connections = set()
+    def add_peer(self, nodeid):
+        entry = (nodeid, "storage", "rref")
+        self._connections.add(entry)
+    def remove_all_peers(self):
+        self._connections.clear()
 
 class Basic(unittest.TestCase):
     def test_loadable(self):
@@ -40,9 +42,9 @@ class Basic(unittest.TestCase):
         secret_fname = os.path.join(basedir, "private", "secret")
         self.failUnless(os.path.exists(secret_fname), secret_fname)
         renew_secret = c.get_renewal_secret()
-        self.failUnless(idlib.b2a(renew_secret))
+        self.failUnless(base32.b2a(renew_secret))
         cancel_secret = c.get_cancel_secret()
-        self.failUnless(idlib.b2a(cancel_secret))
+        self.failUnless(base32.b2a(cancel_secret))
 
     def test_sizelimit_1(self):
         basedir = "client.Basic.test_sizelimit_1"
@@ -51,8 +53,7 @@ class Basic(unittest.TestCase):
         open(os.path.join(basedir, "vdrive.furl"), "w").write("")
         open(os.path.join(basedir, "sizelimit"), "w").write("1000")
         c = client.Client(basedir)
-        self.failUnlessEqual(c.getServiceNamed("storageserver").sizelimit,
-                             1000)
+        self.failUnlessEqual(c.getServiceNamed("storage").sizelimit, 1000)
 
     def test_sizelimit_2(self):
         basedir = "client.Basic.test_sizelimit_2"
@@ -61,8 +62,7 @@ class Basic(unittest.TestCase):
         open(os.path.join(basedir, "vdrive.furl"), "w").write("")
         open(os.path.join(basedir, "sizelimit"), "w").write("10K")
         c = client.Client(basedir)
-        self.failUnlessEqual(c.getServiceNamed("storageserver").sizelimit,
-                             10*1000)
+        self.failUnlessEqual(c.getServiceNamed("storage").sizelimit, 10*1000)
 
     def test_sizelimit_3(self):
         basedir = "client.Basic.test_sizelimit_3"
@@ -71,7 +71,7 @@ class Basic(unittest.TestCase):
         open(os.path.join(basedir, "vdrive.furl"), "w").write("")
         open(os.path.join(basedir, "sizelimit"), "w").write("5mB")
         c = client.Client(basedir)
-        self.failUnlessEqual(c.getServiceNamed("storageserver").sizelimit,
+        self.failUnlessEqual(c.getServiceNamed("storage").sizelimit,
                              5*1000*1000)
 
     def test_sizelimit_4(self):
@@ -81,7 +81,7 @@ class Basic(unittest.TestCase):
         open(os.path.join(basedir, "vdrive.furl"), "w").write("")
         open(os.path.join(basedir, "sizelimit"), "w").write("78Gb")
         c = client.Client(basedir)
-        self.failUnlessEqual(c.getServiceNamed("storageserver").sizelimit,
+        self.failUnlessEqual(c.getServiceNamed("storage").sizelimit,
                              78*1000*1000*1000)
 
     def test_sizelimit_bad(self):
@@ -91,8 +91,11 @@ class Basic(unittest.TestCase):
         open(os.path.join(basedir, "vdrive.furl"), "w").write("")
         open(os.path.join(basedir, "sizelimit"), "w").write("bogus")
         c = client.Client(basedir)
-        self.failUnlessEqual(c.getServiceNamed("storageserver").sizelimit,
-                             None)
+        self.failUnlessEqual(c.getServiceNamed("storage").sizelimit, None)
+
+    def _permute(self, c, key):
+        return [ peerid
+                 for (peerid,rref) in c.get_permuted_peers("storage", key) ]
 
     def test_permute(self):
         basedir = "test_client.Basic.test_permute"
@@ -102,17 +105,18 @@ class Basic(unittest.TestCase):
         c = client.Client(basedir)
         c.introducer_client = FakeIntroducerClient()
         for k in ["%d" % i for i in range(5)]:
-            c.introducer_client.connections[k] = None
-        self.failUnlessEqual(permute(c, "one"), ['3','1','0','4','2'])
-        self.failUnlessEqual(permute(c, "two"), ['0','4','2','1','3'])
-        c.introducer_client.connections.clear()
-        self.failUnlessEqual(permute(c, "one"), [])
+            c.introducer_client.add_peer(k)
+
+        self.failUnlessEqual(self._permute(c, "one"), ['3','1','0','4','2'])
+        self.failUnlessEqual(self._permute(c, "two"), ['0','4','2','1','3'])
+        c.introducer_client.remove_all_peers()
+        self.failUnlessEqual(self._permute(c, "one"), [])
 
         c2 = client.Client(basedir)
         c2.introducer_client = FakeIntroducerClient()
         for k in ["%d" % i for i in range(5)]:
-            c2.introducer_client.connections[k] = None
-        self.failUnlessEqual(permute(c2, "one"), ['3','1','0','4','2'])
+            c2.introducer_client.add_peer(k)
+        self.failUnlessEqual(self._permute(c2, "one"), ['3','1','0','4','2'])
 
     def test_versions(self):
         basedir = "test_client.Basic.test_versions"
@@ -120,7 +124,8 @@ class Basic(unittest.TestCase):
         open(os.path.join(basedir, "introducer.furl"), "w").write("")
         open(os.path.join(basedir, "vdrive.furl"), "w").write("")
         c = client.Client(basedir)
-        mine, oldest = c.remote_get_versions()
+        ss = c.getServiceNamed("storage")
+        mine, oldest = ss.remote_get_versions()
         self.failUnlessEqual(mine, str(allmydata.__version__))
         self.failIfEqual(str(allmydata.__version__), "unknown")
         self.failUnless("." in str(allmydata.__version__),
