@@ -7,6 +7,7 @@ from twisted.internet import defer, reactor
 from twisted.internet import threads # CLI tests use deferToThread
 from twisted.internet.error import ConnectionDone, ConnectionLost
 from twisted.application import service
+import allmydata
 from allmydata import client, uri, download, upload, storage, mutable, offloaded
 from allmydata.introducer import IntroducerNode
 from allmydata.util import deferredutil, fileutil, idlib, mathutil, testutil
@@ -14,8 +15,9 @@ from allmydata.util import log
 from allmydata.scripts import runner
 from allmydata.interfaces import IDirectoryNode, IFileNode, IFileURI
 from allmydata.mutable import NotMutableError
+from allmydata.stats import PickleStatsGatherer
 from foolscap.eventual import flushEventualQueue
-from foolscap import DeadReferenceError
+from foolscap import DeadReferenceError, Tub
 from twisted.python.failure import Failure
 from twisted.web.client import getPage
 from twisted.web.error import Error
@@ -70,12 +72,36 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, unittest.TestCase):
         iv_dir = self.getdir("introducer")
         if not os.path.isdir(iv_dir):
             fileutil.make_dirs(iv_dir)
+        f = open(os.path.join(iv_dir, "webport"), "w")
+        f.write("tcp:0:interface=127.0.0.1\n")
+        f.close()
         iv = IntroducerNode(basedir=iv_dir)
         self.introducer = self.add_service(iv)
         d = self.introducer.when_tub_ready()
+        d.addCallback(self._get_introducer_web)
+        d.addCallback(self._set_up_stats_gatherer)
         d.addCallback(self._set_up_nodes_2)
+        d.addCallback(self._grab_stats)
         return d
 
+    def _get_introducer_web(self, res):
+        f = open(os.path.join(self.getdir("introducer"), "node.url"), "r")
+        self.introweb_url = f.read().strip()
+        f.close()
+
+    def _set_up_stats_gatherer(self, res):
+        statsdir = self.getdir("stats_gatherer")
+        fileutil.make_dirs(statsdir)
+        t = Tub()
+        self.add_service(t)
+        l = t.listenOn("tcp:0")
+        p = l.getPortnum()
+        t.setLocation("localhost:%d" % p)
+
+        self.stats_gatherer = PickleStatsGatherer(t, statsdir, False)
+        self.add_service(self.stats_gatherer)
+        self.stats_gatherer_furl = self.stats_gatherer.get_furl()
+
     def _set_up_nodes_2(self, res):
         q = self.introducer
         self.introducer_furl = q.introducer_url
@@ -96,6 +122,7 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, unittest.TestCase):
                 fileutil.make_dirs(os.path.join(basedir, "private"))
                 open(os.path.join(basedir, "private", "root_dir.cap"), "w")
             open(os.path.join(basedir, "introducer.furl"), "w").write(self.introducer_furl)
+            open(os.path.join(basedir, "stats_gatherer.furl"), "w").write(self.stats_gatherer_furl)
 
         # start client[0], wait for it's tub to be ready (at which point it
         # will have registered the helper furl).
@@ -131,6 +158,10 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, unittest.TestCase):
         d.addCallback(_connected)
         return d
 
+    def _grab_stats(self, res):
+        d = self.stats_gatherer.poll()
+        return d
+
     def bounce_client(self, num):
         c = self.clients[num]
         d = c.disownServiceParent()
@@ -224,15 +255,16 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, unittest.TestCase):
     del test_connections
 
     def test_upload_and_download_random_key(self):
+        self.basedir = "system/SystemTest/test_upload_and_download_random_key"
         return self._test_upload_and_download(False)
     test_upload_and_download_random_key.timeout = 4800
 
     def test_upload_and_download_content_hash_key(self):
+        self.basedir = "system/SystemTest/test_upload_and_download_CHK"
         return self._test_upload_and_download(True)
     test_upload_and_download_content_hash_key.timeout = 4800
 
     def _test_upload_and_download(self, contenthashkey):
-        self.basedir = "system/SystemTest/test_upload_and_download"
         # we use 4000 bytes of data, which will result in about 400k written
         # to disk among all our simulated nodes
         DATA = "Some data to upload\n" * 200
@@ -244,6 +276,7 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, unittest.TestCase):
                 permuted_peers = list(c.get_permuted_peers("storage", "a"))
                 self.failUnlessEqual(len(permuted_peers), self.numclients)
         d.addCallback(_check_connections)
+
         def _do_upload(res):
             log.msg("UPLOADING")
             u = self.clients[0].getServiceNamed("uploader")
@@ -662,7 +695,7 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, unittest.TestCase):
             self.failUnlessEqual(res, DATA)
             # replace the data
             log.msg("starting replace1")
-            d1 = newnode.replace(NEWDATA)
+            d1 = newnode.update(NEWDATA)
             d1.addCallback(lambda res: newnode.download_to_data())
             return d1
         d.addCallback(_check_download_3)
@@ -676,7 +709,7 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, unittest.TestCase):
             newnode2 = self.clients[3].create_node_from_uri(uri)
             self._newnode3 = self.clients[3].create_node_from_uri(uri)
             log.msg("starting replace2")
-            d1 = newnode1.replace(NEWERDATA)
+            d1 = newnode1.overwrite(NEWERDATA)
             d1.addCallback(lambda res: newnode2.download_to_data())
             return d1
         d.addCallback(_check_download_4)
@@ -799,6 +832,7 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, unittest.TestCase):
         self.basedir = "system/SystemTest/test_vdrive"
         self.data = LARGE_DATA
         d = self.set_up_nodes(createprivdir=True)
+        d.addCallback(self._test_introweb)
         d.addCallback(self.log, "starting publish")
         d.addCallback(self._do_publish1)
         d.addCallback(self._test_runner)
@@ -836,9 +870,25 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, unittest.TestCase):
         # P/test_put/  (empty)
         d.addCallback(self._test_checker)
         d.addCallback(self._test_verifier)
+        d.addCallback(self._grab_stats)
         return d
     test_vdrive.timeout = 1100
 
+    def _test_introweb(self, res):
+        d = getPage(self.introweb_url, method="GET", followRedirect=True)
+        def _check(res):
+            try:
+                self.failUnless("allmydata: %s" % str(allmydata.__version__)
+                                in res)
+                self.failUnless("Summary: storage: 5, stub_client: 5" in res)
+            except unittest.FailTest:
+                print
+                print "GET %s output was:" % self.introweb_url
+                print res
+                raise
+        d.addCallback(_check)
+        return d
+
     def _do_publish1(self, res):
         ut = upload.Data(self.data)
         c0 = self.clients[0]
@@ -1171,7 +1221,33 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, unittest.TestCase):
                                             file=("foo.txt", "data2" * 10000)))
 
         # check that the status page exists
-        d.addCallback(lambda res: self.GET("status"))
+        d.addCallback(lambda res: self.GET("status", followRedirect=True))
+        def _got_status(res):
+            # find an interesting upload and download to look at. LIT files
+            # are not interesting.
+            for dl in self.clients[0].list_recent_downloads():
+                if dl.get_size() > 200:
+                    self._down_status = dl.get_counter()
+            for ul in self.clients[0].list_recent_uploads():
+                if ul.get_size() > 200:
+                    self._up_status = ul.get_counter()
+            rs = self.clients[0].list_recent_retrieve()[0]
+            self._retrieve_status = rs.get_counter()
+            ps = self.clients[0].list_recent_publish()[0]
+            self._publish_status = ps.get_counter()
+
+            # and that there are some upload- and download- status pages
+            return self.GET("status/up-%d" % self._up_status)
+        d.addCallback(_got_status)
+        def _got_up(res):
+            return self.GET("status/down-%d" % self._down_status)
+        d.addCallback(_got_up)
+        def _got_down(res):
+            return self.GET("status/publish-%d" % self._publish_status)
+        d.addCallback(_got_down)
+        def _got_publish(res):
+            return self.GET("status/retrieve-%d" % self._retrieve_status)
+        d.addCallback(_got_publish)
 
         # TODO: mangle the second segment of a file, to test errors that
         # occur after we've already sent some good data, which uses a
