@@ -2,14 +2,20 @@
 import os
 from twisted.trial import unittest
 from twisted.python.failure import Failure
-from twisted.internet import defer
+from twisted.python import log
 from cStringIO import StringIO
 
 from allmydata import upload, encode, uri
 from allmydata.interfaces import IFileURI
 from allmydata.util.assertutil import precondition
+from allmydata.util.deferredutil import DeferredListShouldSucceed
 from foolscap import eventual
 
+MiB = 1024*1024
+
+def extract_uri(results):
+    return results.uri
+
 class Uploadable(unittest.TestCase):
     def shouldEqual(self, data, expected):
         self.failUnless(isinstance(data, list))
@@ -18,9 +24,15 @@ class Uploadable(unittest.TestCase):
         s = "".join(data)
         self.failUnlessEqual(s, expected)
 
-    def test_filehandle(self):
+    def test_filehandle_random_key(self):
+        return self._test_filehandle(True)
+
+    def test_filehandle_content_hash_key(self):
+        return self._test_filehandle(False)
+
+    def _test_filehandle(self, randomkey):
         s = StringIO("a"*41)
-        u = upload.FileHandle(s)
+        u = upload.FileHandle(s, randomkey)
         d = u.get_size()
         d.addCallback(self.failUnlessEqual, 41)
         d.addCallback(lambda res: u.read(1))
@@ -60,20 +72,6 @@ class Uploadable(unittest.TestCase):
         d.addCallback(lambda res: u.close())
         return d
 
-class FakePeer:
-    def __init__(self, mode="good"):
-        self.ss = FakeStorageServer(mode)
-
-    def callRemote(self, methname, *args, **kwargs):
-        def _call():
-            meth = getattr(self, methname)
-            return meth(*args, **kwargs)
-        return defer.maybeDeferred(_call)
-
-    def get_service(self, sname):
-        assert sname == "storageserver"
-        return self.ss
-
 class FakeStorageServer:
     def __init__(self, mode):
         self.mode = mode
@@ -131,26 +129,27 @@ class FakeBucketWriter:
         precondition(not self.closed)
         self.closed = True
 
-class FakeIntroducerClient:
-    def when_enough_peers(self, numpeers):
-        return defer.succeed(None)
+    def remote_abort(self):
+        log.err("uh oh, I was asked to abort")
 
 class FakeClient:
+    DEFAULT_ENCODING_PARAMETERS = {"k":25,
+                                   "happy": 75,
+                                   "n": 100,
+                                   "max_segment_size": 1*MiB,
+                                   }
     def __init__(self, mode="good", num_servers=50):
         self.mode = mode
         self.num_servers = num_servers
-        self.introducer_client = FakeIntroducerClient()
     def log(self, *args, **kwargs):
         pass
     def get_permuted_peers(self, storage_index, include_myself):
-        peers = [ ("%20d"%fakeid, "%20d"%fakeid, FakePeer(self.mode),)
+        peers = [ ("%20d"%fakeid, FakeStorageServer(self.mode),)
                   for fakeid in range(self.num_servers) ]
-        self.last_peers = [p[2] for p in peers]
+        self.last_peers = [p[1] for p in peers]
         return peers
-    def get_push_to_ourselves(self):
-        return None
     def get_encoding_parameters(self):
-        return None
+        return self.DEFAULT_ENCODING_PARAMETERS
 
     def get_renewal_secret(self):
         return ""
@@ -169,6 +168,16 @@ SIZE_ZERO = 0
 SIZE_SMALL = 16
 SIZE_LARGE = len(DATA)
 
+def upload_data(uploader, data):
+    u = upload.Data(data)
+    return uploader.upload(u)
+def upload_filename(uploader, filename):
+    u = upload.FileName(filename)
+    return uploader.upload(u)
+def upload_filehandle(uploader, fh):
+    u = upload.FileHandle(fh)
+    return uploader.upload(u)
+
 class GoodServer(unittest.TestCase):
     def setUp(self):
         self.node = FakeClient(mode="good")
@@ -176,6 +185,14 @@ class GoodServer(unittest.TestCase):
         self.u.running = True
         self.u.parent = self.node
 
+    def set_encoding_parameters(self, k, happy, n, max_segsize=1*MiB):
+        p = {"k": k,
+             "happy": happy,
+             "n": n,
+             "max_segment_size": max_segsize,
+             }
+        self.node.DEFAULT_ENCODING_PARAMETERS = p
+
     def _check_small(self, newuri, size):
         u = IFileURI(newuri)
         self.failUnless(isinstance(u, uri.LiteralFileURI))
@@ -195,19 +212,22 @@ class GoodServer(unittest.TestCase):
 
     def test_data_zero(self):
         data = self.get_data(SIZE_ZERO)
-        d = self.u.upload_data(data)
+        d = upload_data(self.u, data)
+        d.addCallback(extract_uri)
         d.addCallback(self._check_small, SIZE_ZERO)
         return d
 
     def test_data_small(self):
         data = self.get_data(SIZE_SMALL)
-        d = self.u.upload_data(data)
+        d = upload_data(self.u, data)
+        d.addCallback(extract_uri)
         d.addCallback(self._check_small, SIZE_SMALL)
         return d
 
     def test_data_large(self):
         data = self.get_data(SIZE_LARGE)
-        d = self.u.upload_data(data)
+        d = upload_data(self.u, data)
+        d.addCallback(extract_uri)
         d.addCallback(self._check_large, SIZE_LARGE)
         return d
 
@@ -215,25 +235,30 @@ class GoodServer(unittest.TestCase):
         data = self.get_data(SIZE_LARGE)
         segsize = int(SIZE_LARGE / 2.5)
         # we want 3 segments, since that's not a power of two
-        d = self.u.upload_data(data, {"max_segment_size": segsize})
+        self.set_encoding_parameters(25, 75, 100, segsize)
+        d = upload_data(self.u, data)
+        d.addCallback(extract_uri)
         d.addCallback(self._check_large, SIZE_LARGE)
         return d
 
     def test_filehandle_zero(self):
         data = self.get_data(SIZE_ZERO)
-        d = self.u.upload_filehandle(StringIO(data))
+        d = upload_filehandle(self.u, StringIO(data))
+        d.addCallback(extract_uri)
         d.addCallback(self._check_small, SIZE_ZERO)
         return d
 
     def test_filehandle_small(self):
         data = self.get_data(SIZE_SMALL)
-        d = self.u.upload_filehandle(StringIO(data))
+        d = upload_filehandle(self.u, StringIO(data))
+        d.addCallback(extract_uri)
         d.addCallback(self._check_small, SIZE_SMALL)
         return d
 
     def test_filehandle_large(self):
         data = self.get_data(SIZE_LARGE)
-        d = self.u.upload_filehandle(StringIO(data))
+        d = upload_filehandle(self.u, StringIO(data))
+        d.addCallback(extract_uri)
         d.addCallback(self._check_large, SIZE_LARGE)
         return d
 
@@ -243,7 +268,8 @@ class GoodServer(unittest.TestCase):
         data = self.get_data(SIZE_ZERO)
         f.write(data)
         f.close()
-        d = self.u.upload_filename(fn)
+        d = upload_filename(self.u, fn)
+        d.addCallback(extract_uri)
         d.addCallback(self._check_small, SIZE_ZERO)
         return d
 
@@ -253,7 +279,8 @@ class GoodServer(unittest.TestCase):
         data = self.get_data(SIZE_SMALL)
         f.write(data)
         f.close()
-        d = self.u.upload_filename(fn)
+        d = upload_filename(self.u, fn)
+        d.addCallback(extract_uri)
         d.addCallback(self._check_small, SIZE_SMALL)
         return d
 
@@ -263,7 +290,8 @@ class GoodServer(unittest.TestCase):
         data = self.get_data(SIZE_LARGE)
         f.write(data)
         f.close()
-        d = self.u.upload_filename(fn)
+        d = upload_filename(self.u, fn)
+        d.addCallback(extract_uri)
         d.addCallback(self._check_large, SIZE_LARGE)
         return d
 
@@ -279,7 +307,7 @@ class FullServer(unittest.TestCase):
 
     def test_data_large(self):
         data = DATA
-        d = self.u.upload_data(data)
+        d = upload_data(self.u, data)
         d.addBoth(self._should_fail)
         return d
 
@@ -303,20 +331,29 @@ class PeerSelection(unittest.TestCase):
         self.failUnlessEqual(len(u.key), 16)
         self.failUnlessEqual(u.size, size)
 
+    def set_encoding_parameters(self, k, happy, n, max_segsize=1*MiB):
+        p = {"k": k,
+             "happy": happy,
+             "n": n,
+             "max_segment_size": max_segsize,
+             }
+        self.node.DEFAULT_ENCODING_PARAMETERS = p
+
     def test_one_each(self):
         # if we have 50 shares, and there are 50 peers, and they all accept a
         # share, we should get exactly one share per peer
 
         self.make_client()
         data = self.get_data(SIZE_LARGE)
-        self.u.DEFAULT_ENCODING_PARAMETERS = (25, 30, 50)
-        d = self.u.upload_data(data)
+        self.set_encoding_parameters(25, 30, 50)
+        d = upload_data(self.u, data)
+        d.addCallback(extract_uri)
         d.addCallback(self._check_large, SIZE_LARGE)
         def _check(res):
             for p in self.node.last_peers:
-                allocated = p.ss.allocated
+                allocated = p.allocated
                 self.failUnlessEqual(len(allocated), 1)
-                self.failUnlessEqual(p.ss.queries, 1)
+                self.failUnlessEqual(p.queries, 1)
         d.addCallback(_check)
         return d
 
@@ -326,14 +363,15 @@ class PeerSelection(unittest.TestCase):
 
         self.make_client()
         data = self.get_data(SIZE_LARGE)
-        self.u.DEFAULT_ENCODING_PARAMETERS = (50, 75, 100)
-        d = self.u.upload_data(data)
+        self.set_encoding_parameters(50, 75, 100)
+        d = upload_data(self.u, data)
+        d.addCallback(extract_uri)
         d.addCallback(self._check_large, SIZE_LARGE)
         def _check(res):
             for p in self.node.last_peers:
-                allocated = p.ss.allocated
+                allocated = p.allocated
                 self.failUnlessEqual(len(allocated), 2)
-                self.failUnlessEqual(p.ss.queries, 2)
+                self.failUnlessEqual(p.queries, 2)
         d.addCallback(_check)
         return d
 
@@ -343,20 +381,21 @@ class PeerSelection(unittest.TestCase):
 
         self.make_client()
         data = self.get_data(SIZE_LARGE)
-        self.u.DEFAULT_ENCODING_PARAMETERS = (24, 41, 51)
-        d = self.u.upload_data(data)
+        self.set_encoding_parameters(24, 41, 51)
+        d = upload_data(self.u, data)
+        d.addCallback(extract_uri)
         d.addCallback(self._check_large, SIZE_LARGE)
         def _check(res):
             got_one = []
             got_two = []
             for p in self.node.last_peers:
-                allocated = p.ss.allocated
+                allocated = p.allocated
                 self.failUnless(len(allocated) in (1,2), len(allocated))
                 if len(allocated) == 1:
-                    self.failUnlessEqual(p.ss.queries, 1)
+                    self.failUnlessEqual(p.queries, 1)
                     got_one.append(p)
                 else:
-                    self.failUnlessEqual(p.ss.queries, 2)
+                    self.failUnlessEqual(p.queries, 2)
                     got_two.append(p)
             self.failUnlessEqual(len(got_one), 49)
             self.failUnlessEqual(len(got_two), 1)
@@ -370,14 +409,15 @@ class PeerSelection(unittest.TestCase):
 
         self.make_client()
         data = self.get_data(SIZE_LARGE)
-        self.u.DEFAULT_ENCODING_PARAMETERS = (100, 150, 200)
-        d = self.u.upload_data(data)
+        self.set_encoding_parameters(100, 150, 200)
+        d = upload_data(self.u, data)
+        d.addCallback(extract_uri)
         d.addCallback(self._check_large, SIZE_LARGE)
         def _check(res):
             for p in self.node.last_peers:
-                allocated = p.ss.allocated
+                allocated = p.allocated
                 self.failUnlessEqual(len(allocated), 4)
-                self.failUnlessEqual(p.ss.queries, 2)
+                self.failUnlessEqual(p.queries, 2)
         d.addCallback(_check)
         return d
 
@@ -387,19 +427,58 @@ class PeerSelection(unittest.TestCase):
 
         self.make_client(3)
         data = self.get_data(SIZE_LARGE)
-        self.u.DEFAULT_ENCODING_PARAMETERS = (3, 5, 10)
-        d = self.u.upload_data(data)
+        self.set_encoding_parameters(3, 5, 10)
+        d = upload_data(self.u, data)
+        d.addCallback(extract_uri)
         d.addCallback(self._check_large, SIZE_LARGE)
         def _check(res):
             counts = {}
             for p in self.node.last_peers:
-                allocated = p.ss.allocated
+                allocated = p.allocated
                 counts[len(allocated)] = counts.get(len(allocated), 0) + 1
             histogram = [counts.get(i, 0) for i in range(5)]
             self.failUnlessEqual(histogram, [0,0,0,2,1])
         d.addCallback(_check)
         return d
 
+class StorageIndex(unittest.TestCase):
+    def test_params_must_matter(self):
+        DATA = "I am some data"
+        u = upload.Data(DATA)
+        eu = upload.EncryptAnUploadable(u)
+        d1 = eu.get_storage_index()
+
+        # CHK means the same data should encrypt the same way
+        u = upload.Data(DATA)
+        eu = upload.EncryptAnUploadable(u)
+        d1a = eu.get_storage_index()
+
+        # but if we change the encoding parameters, it should be different
+        u = upload.Data(DATA)
+        u.encoding_param_k = u.default_encoding_param_k + 1
+        eu = upload.EncryptAnUploadable(u)
+        d2 = eu.get_storage_index()
+
+        # and if we use a random key, it should be different than the CHK
+        u = upload.Data(DATA, contenthashkey=False)
+        eu = upload.EncryptAnUploadable(u)
+        d3 = eu.get_storage_index()
+        # and different from another instance
+        u = upload.Data(DATA, contenthashkey=False)
+        eu = upload.EncryptAnUploadable(u)
+        d4 = eu.get_storage_index()
+
+        d = DeferredListShouldSucceed([d1,d1a,d2,d3,d4])
+        def _done(res):
+            si1, si1a, si2, si3, si4 = res
+            self.failUnlessEqual(si1, si1a)
+            self.failIfEqual(si1, si2)
+            self.failIfEqual(si1, si3)
+            self.failIfEqual(si1, si4)
+            self.failIfEqual(si3, si4)
+        d.addCallback(_done)
+        return d
+
 
 # TODO:
 #  upload with exactly 75 peers (shares_of_happiness)
