@@ -6,8 +6,9 @@ from twisted.internet import defer
 from twisted.web import client, error, http
 from twisted.python import failure, log
 from allmydata import interfaces, provisioning, uri, webish, upload, download
+from allmydata.web import status
 from allmydata.util import fileutil
-from allmydata.test.common import NonGridDirectoryNode, FakeCHKFileNode, FakeMutableFileNode, create_chk_filenode
+from allmydata.test.common import FakeDirectoryNode, FakeCHKFileNode, FakeMutableFileNode, create_chk_filenode
 from allmydata.interfaces import IURI, INewDirectoryURI, IReadonlyNewDirectoryURI, IFileURI, IMutableFileURI, IMutableFileNode
 
 # create a fake uploader/downloader, and a couple of fake dirnodes, then
@@ -32,21 +33,24 @@ class FakeClient(service.MultiService):
                 }
     introducer_furl = "None"
     introducer_client = FakeIntroducerClient()
+    _all_upload_status = [upload.UploadStatus()]
+    _all_download_status = [download.DownloadStatus()]
+
     def connected_to_introducer(self):
         return False
 
-    def create_node_from_uri(self, uri):
-        u = IURI(uri)
+    def create_node_from_uri(self, auri):
+        u = uri.from_string(auri)
         if (INewDirectoryURI.providedBy(u)
             or IReadonlyNewDirectoryURI.providedBy(u)):
-            return NonGridDirectoryNode(self).init_from_uri(u)
+            return FakeDirectoryNode(self).init_from_uri(u)
         if IFileURI.providedBy(u):
             return FakeCHKFileNode(u, self)
         assert IMutableFileURI.providedBy(u), u
         return FakeMutableFileNode(self).init_from_uri(u)
 
     def create_empty_dirnode(self):
-        n = NonGridDirectoryNode(self)
+        n = FakeDirectoryNode(self)
         d = n.create()
         d.addCallback(lambda res: n)
         return d
@@ -67,10 +71,28 @@ class FakeClient(service.MultiService):
         d.addCallback(_got_data)
         return d
 
-    def list_uploads(self):
-        return [upload.UploadStatus()]
-    def list_downloads(self):
-        return [download.DownloadStatus()]
+    def list_all_uploads(self):
+        return []
+    def list_all_downloads(self):
+        return []
+
+    def list_active_uploads(self):
+        return self._all_upload_status
+    def list_active_downloads(self):
+        return self._all_download_status
+    def list_active_publish(self):
+        return []
+    def list_active_retrieve(self):
+        return []
+
+    def list_recent_uploads(self):
+        return self._all_upload_status
+    def list_recent_downloads(self):
+        return self._all_download_status
+    def list_recent_publish(self):
+        return []
+    def list_recent_retrieve(self):
+        return []
 
 
 class WebMixin(object):
@@ -375,12 +397,47 @@ class Web(WebMixin, unittest.TestCase):
         return d
 
     def test_status(self):
-        d = self.GET("/status")
+        dl_num = self.s.list_recent_downloads()[0].get_counter()
+        ul_num = self.s.list_recent_uploads()[0].get_counter()
+        d = self.GET("/status", followRedirect=True)
         def _check(res):
-            self.failUnless('Current Uploads and Downloads' in res)
+            self.failUnless('Upload and Download Status' in res, res)
+            self.failUnless('"down-%d"' % dl_num in res, res)
+            self.failUnless('"up-%d"' % ul_num in res, res)
         d.addCallback(_check)
+        d.addCallback(lambda res: self.GET("/status/down-%d" % dl_num))
+        def _check_dl(res):
+            self.failUnless("File Download Status" in res, res)
+        d.addCallback(_check_dl)
+        d.addCallback(lambda res: self.GET("/status/up-%d" % ul_num))
+        def _check_ul(res):
+            self.failUnless("File Upload Status" in res, res)
+        d.addCallback(_check_ul)
         return d
 
+    def test_status_numbers(self):
+        drrm = status.DownloadResultsRendererMixin()
+        self.failUnlessEqual(drrm.render_time(None, None), "")
+        self.failUnlessEqual(drrm.render_time(None, 2.5), "2.50s")
+        self.failUnlessEqual(drrm.render_time(None, 0.25), "250ms")
+        self.failUnlessEqual(drrm.render_time(None, 0.0021), "2.1ms")
+        self.failUnlessEqual(drrm.render_time(None, 0.000123), "123us")
+        self.failUnlessEqual(drrm.render_rate(None, None), "")
+        self.failUnlessEqual(drrm.render_rate(None, 2500000), "2.50MBps")
+        self.failUnlessEqual(drrm.render_rate(None, 30100), "30.1kBps")
+        self.failUnlessEqual(drrm.render_rate(None, 123), "123Bps")
+
+        urrm = status.UploadResultsRendererMixin()
+        self.failUnlessEqual(urrm.render_time(None, None), "")
+        self.failUnlessEqual(urrm.render_time(None, 2.5), "2.50s")
+        self.failUnlessEqual(urrm.render_time(None, 0.25), "250ms")
+        self.failUnlessEqual(urrm.render_time(None, 0.0021), "2.1ms")
+        self.failUnlessEqual(urrm.render_time(None, 0.000123), "123us")
+        self.failUnlessEqual(urrm.render_rate(None, None), "")
+        self.failUnlessEqual(urrm.render_rate(None, 2500000), "2.50MBps")
+        self.failUnlessEqual(urrm.render_rate(None, 30100), "30.1kBps")
+        self.failUnlessEqual(urrm.render_rate(None, 123), "123Bps")
+
     def test_GET_FILEURL(self):
         d = self.GET(self.public_url + "/foo/bar.txt")
         d.addCallback(self.failUnlessIsBarDotTxt)
@@ -1299,6 +1356,46 @@ class Web(WebMixin, unittest.TestCase):
         d.addCallback(self.failUnlessNodeKeysAre, [])
         return d
 
+    def test_POST_set_children(self):
+        contents9, n9, newuri9 = self.makefile(9)
+        contents10, n10, newuri10 = self.makefile(10)
+        contents11, n11, newuri11 = self.makefile(11)
+
+        reqbody = """{
+                     "atomic_added_1": [ "filenode", { "rw_uri": "%s",
+                                                "size": 0,
+                                                "metadata": {
+                                                  "ctime": 1002777696.7564139,
+                                                  "mtime": 1002777696.7564139
+                                                 }
+                                               } ],
+                     "atomic_added_2": [ "filenode", { "rw_uri": "%s",
+                                                "size": 1,
+                                                "metadata": {
+                                                  "ctime": 1002777696.7564139,
+                                                  "mtime": 1002777696.7564139
+                                                 }
+                                               } ],
+                     "atomic_added_3": [ "filenode", { "rw_uri": "%s",
+                                                "size": 2,
+                                                "metadata": {
+                                                  "ctime": 1002777696.7564139,
+                                                  "mtime": 1002777696.7564139
+                                                 }
+                                               } ]
+                    }""" % (newuri9, newuri10, newuri11)
+
+        url = self.webish_url + self.public_url + "/foo" + "?t=set_children"
+
+        d = client.getPage(url, method="POST", postdata=reqbody)
+        def _then(res):
+            self.failUnlessURIMatchesChild(newuri9, self._foo_node, u"atomic_added_1")
+            self.failUnlessURIMatchesChild(newuri10, self._foo_node, u"atomic_added_2")
+            self.failUnlessURIMatchesChild(newuri11, self._foo_node, u"atomic_added_3")
+
+        d.addCallback(_then)
+        return d
+
     def test_POST_put_uri(self):
         contents, n, newuri = self.makefile(8)
         d = self.POST(self.public_url + "/foo", t="uri", name="new.txt", uri=newuri)
