@@ -1,5 +1,5 @@
 
-import os, time, weakref
+import os, time, weakref, itertools
 from zope.interface import implements
 from twisted.python import failure
 from twisted.internet import defer
@@ -41,13 +41,16 @@ class UploadResults(Copyable, RemoteCopy):
     typeToCopy = "allmydata.upload.UploadResults.tahoe.allmydata.com"
     copytype = typeToCopy
 
-    file_size = None
-    ciphertext_fetched = None # how much the helper fetched
-    uri = None
-    sharemap = None # dict of shnum to placement string
-    servermap = None # dict of peerid to set(shnums)
     def __init__(self):
         self.timings = {} # dict of name to number of seconds
+        self.sharemap = {} # dict of shnum to placement string
+        self.servermap = {} # dict of peerid to set(shnums)
+        self.file_size = None
+        self.ciphertext_fetched = None # how much the helper fetched
+        self.uri = None
+        self.preexisting_shares = None # count of shares already present
+        self.pushed_shares = None # count of shares we pushed
+
 
 # our current uri_extension is 846 bytes for small files, a few bytes
 # more for larger ones (since the filesize is encoded in decimal in a
@@ -129,8 +132,11 @@ class Tahoe2PeerSelector:
                          storage_index, share_size, block_size,
                          num_segments, total_shares, shares_of_happiness):
         """
-        @return: a set of PeerTracker instances that have agreed to hold some
-                 shares for us
+        @return: (used_peers, already_peers), where used_peers is a set of
+                 PeerTracker instances that have agreed to hold some shares
+                 for us (the shnum is stashed inside the PeerTracker),
+                 and already_peers is a dict mapping shnum to a peer
+                 which claims to already have the share.
         """
 
         if self._status:
@@ -145,7 +151,7 @@ class Tahoe2PeerSelector:
         self.contacted_peers2 = [] # peers that we have asked again
         self._started_second_pass = False
         self.use_peers = set() # PeerTrackers that have shares assigned to them
-        self.preexisting_shares = {} # sharenum -> PeerTracker holding the share
+        self.preexisting_shares = {} # sharenum -> peerid holding the share
 
         peers = client.get_permuted_peers("storage", storage_index)
         if not peers:
@@ -198,7 +204,7 @@ class Tahoe2PeerSelector:
                     self.error_count))
             log.msg("peer selection successful for %s: %s" % (self, msg),
                     parent=self._log_parent)
-            return self.use_peers
+            return (self.use_peers, self.preexisting_shares)
 
         if self.uncontacted_peers:
             peer = self.uncontacted_peers.pop(0)
@@ -297,7 +303,7 @@ class Tahoe2PeerSelector:
                     level=log.NOISY, parent=self._log_parent)
             progress = False
             for s in alreadygot:
-                self.preexisting_shares[s] = peer
+                self.preexisting_shares[s] = peer.peerid
                 if s in self.homeless_shares:
                     self.homeless_shares.remove(s)
                     progress = True
@@ -347,8 +353,9 @@ class EncryptAnUploadable:
     implements(IEncryptedUploadable)
     CHUNKSIZE = 50*1024
 
-    def __init__(self, original):
+    def __init__(self, original, log_parent=None):
         self.original = IUploadable(original)
+        self._log_number = log_parent
         self._encryptor = None
         self._plaintext_hasher = plaintext_hasher()
         self._plaintext_segment_hasher = None
@@ -365,6 +372,8 @@ class EncryptAnUploadable:
     def log(self, *args, **kwargs):
         if "facility" not in kwargs:
             kwargs["facility"] = "upload.encryption"
+        if "parent" not in kwargs:
+            kwargs["parent"] = self._log_number
         return log.msg(*args, **kwargs)
 
     def get_size(self):
@@ -511,8 +520,8 @@ class EncryptAnUploadable:
         bytes_processed = 0
         while data:
             chunk = data.pop(0)
-            log.msg(" read_encrypted handling %dB-sized chunk" % len(chunk),
-                    level=log.NOISY)
+            self.log(" read_encrypted handling %dB-sized chunk" % len(chunk),
+                     level=log.NOISY)
             bytes_processed += len(chunk)
             self._plaintext_hasher.update(chunk)
             self._update_segment_hash(chunk)
@@ -523,7 +532,7 @@ class EncryptAnUploadable:
             # before each call to (hash_only==False) _encryptor.process()
             ciphertext = self._encryptor.process(chunk)
             if hash_only:
-                log.msg("  skipping encryption")
+                self.log("  skipping encryption", level=log.NOISY)
             else:
                 cryptdata.append(ciphertext)
             del ciphertext
@@ -561,6 +570,7 @@ class EncryptAnUploadable:
 
 class UploadStatus:
     implements(IUploadStatus)
+    statusid_counter = itertools.count(0)
 
     def __init__(self):
         self.storage_index = None
@@ -568,7 +578,13 @@ class UploadStatus:
         self.helper = False
         self.status = "Not started"
         self.progress = [0.0, 0.0, 0.0]
+        self.active = True
+        self.results = None
+        self.counter = self.statusid_counter.next()
+        self.started = time.time()
 
+    def get_started(self):
+        return self.started
     def get_storage_index(self):
         return self.storage_index
     def get_size(self):
@@ -579,6 +595,12 @@ class UploadStatus:
         return self.status
     def get_progress(self):
         return tuple(self.progress)
+    def get_active(self):
+        return self.active
+    def get_results(self):
+        return self.results
+    def get_counter(self):
+        return self.counter
 
     def set_storage_index(self, si):
         self.storage_index = si
@@ -591,6 +613,10 @@ class UploadStatus:
     def set_progress(self, which, value):
         # [0]: chk, [1]: ciphertext, [2]: encode+push
         self.progress[which] = value
+    def set_active(self, value):
+        self.active = value
+    def set_results(self, value):
+        self.results = value
 
 class CHKUploader:
     peer_selector_class = Tahoe2PeerSelector
@@ -603,6 +629,8 @@ class CHKUploader:
         self._storage_index = None
         self._upload_status = UploadStatus()
         self._upload_status.set_helper(False)
+        self._upload_status.set_active(True)
+        self._upload_status.set_results(self._results)
 
     def log(self, *args, **kwargs):
         if "parent" not in kwargs:
@@ -621,7 +649,7 @@ class CHKUploader:
         uploadable = IUploadable(uploadable)
         self.log("starting upload of %s" % uploadable)
 
-        eu = EncryptAnUploadable(uploadable)
+        eu = EncryptAnUploadable(uploadable, self._log_number)
         eu.set_upload_status(self._upload_status)
         d = self.start_encrypted(eu)
         def _uploaded(res):
@@ -629,6 +657,10 @@ class CHKUploader:
             d1.addCallback(lambda key: self._compute_uri(res, key))
             return d1
         d.addCallback(_uploaded)
+        def _done(res):
+            self._upload_status.set_active(False)
+            return res
+        d.addBoth(_done)
         return d
 
     def abort(self):
@@ -679,11 +711,22 @@ class CHKUploader:
         d.addCallback(_done)
         return d
 
-    def set_shareholders(self, used_peers, encoder):
+    def set_shareholders(self, (used_peers, already_peers), encoder):
         """
         @param used_peers: a sequence of PeerTracker objects
+        @paran already_peers: a dict mapping sharenum to a peerid that
+                              claims to already have this share
         """
         self.log("_send_shares, used_peers is %s" % (used_peers,))
+        # record already-present shares in self._results
+        for (shnum, peerid) in already_peers.items():
+            peerid_s = idlib.shortnodeid_b2a(peerid)
+            self._results.sharemap[shnum] = "Found on [%s]" % peerid_s
+            if peerid not in self._results.servermap:
+                self._results.servermap[peerid] = set()
+            self._results.servermap[peerid].add(shnum)
+        self._results.preexisting_shares = len(already_peers)
+
         self._sharemap = {}
         for peer in used_peers:
             assert isinstance(peer, PeerTracker)
@@ -697,8 +740,6 @@ class CHKUploader:
 
     def _encrypted_done(self, res):
         r = self._results
-        r.sharemap = {}
-        r.servermap = {}
         for shnum in self._encoder.get_shares_placed():
             peer_tracker = self._sharemap[shnum]
             peerid = peer_tracker.peerid
@@ -707,6 +748,7 @@ class CHKUploader:
             if peerid not in r.servermap:
                 r.servermap[peerid] = set()
             r.servermap[peerid].add(shnum)
+        r.pushed_shares = len(self._encoder.get_shares_placed())
         now = time.time()
         r.file_size = self._encoder.file_size
         r.timings["total"] = now - self._started
@@ -758,6 +800,8 @@ class LiteralUploader:
         s.set_storage_index(None)
         s.set_helper(False)
         s.set_progress(0, 1.0)
+        s.set_active(False)
+        s.set_results(self._results)
 
     def start(self, uploadable):
         uploadable = IUploadable(uploadable)
@@ -874,16 +918,17 @@ class AssistedUploader:
         self._storage_index = None
         self._upload_status = s = UploadStatus()
         s.set_helper(True)
+        s.set_active(True)
 
-    def log(self, msg, parent=None, **kwargs):
-        if parent is None:
-            parent = self._log_number
-        return log.msg(msg, parent=parent, **kwargs)
+    def log(self, *args, **kwargs):
+        if "parent" not in kwargs:
+            kwargs["parent"] = self._log_number
+        return log.msg(*args, **kwargs)
 
     def start(self, uploadable):
         self._started = time.time()
         u = IUploadable(uploadable)
-        eu = EncryptAnUploadable(u)
+        eu = EncryptAnUploadable(u, self._log_number)
         eu.set_upload_status(self._upload_status)
         self._encuploadable = eu
         d = eu.get_size()
@@ -900,6 +945,10 @@ class AssistedUploader:
         d.addCallback(self._got_storage_index)
         d.addCallback(self._contact_helper)
         d.addCallback(self._build_readcap)
+        def _done(res):
+            self._upload_status.set_active(False)
+            return res
+        d.addBoth(_done)
         return d
 
     def _got_size(self, size):
@@ -923,7 +972,8 @@ class AssistedUploader:
     def _contact_helper(self, res):
         now = self._time_contacting_helper_start = time.time()
         self._storage_index_elapsed = now - self._started
-        self.log("contacting helper..")
+        self.log(format="contacting helper for SI %(si)s..",
+                 si=storage.si_b2a(self._storage_index))
         self._upload_status.set_status("Contacting Helper")
         d = self._helper.callRemote("upload_chk", self._storage_index)
         d.addCallback(self._contacted_helper)
@@ -947,6 +997,7 @@ class AssistedUploader:
             return d
         self.log("helper says file is already uploaded")
         self._upload_status.set_progress(1, 1.0)
+        self._upload_status.set_results(upload_results)
         return upload_results
 
     def _build_readcap(self, upload_results):
@@ -972,13 +1023,14 @@ class AssistedUploader:
             r.timings["helper_total"] = r.timings["total"]
         r.timings["total"] = now - self._started
         self._upload_status.set_status("Done")
+        self._upload_status.set_results(r)
         return r
 
     def get_upload_status(self):
         return self._upload_status
 
 class BaseUploadable:
-    default_max_segment_size = 1*MiB # overridden by max_segment_size
+    default_max_segment_size = 128*KiB # overridden by max_segment_size
     default_encoding_param_k = 3 # overridden by encoding_parameters
     default_encoding_param_happy = 7
     default_encoding_param_n = 10
@@ -1112,17 +1164,20 @@ class Data(FileHandle):
         FileHandle.__init__(self, StringIO(data), contenthashkey=contenthashkey)
 
 class Uploader(service.MultiService):
-    """I am a service that allows file uploading.
+    """I am a service that allows file uploading. I am a service-child of the
+    Client.
     """
     implements(IUploader)
     name = "uploader"
     uploader_class = CHKUploader
     URI_LIT_SIZE_THRESHOLD = 55
+    MAX_UPLOAD_STATUSES = 10
 
     def __init__(self, helper_furl=None):
         self._helper_furl = helper_furl
         self._helper = None
         self._all_uploads = weakref.WeakKeyDictionary()
+        self._recent_upload_status = []
         service.MultiService.__init__(self)
 
     def startService(self):
@@ -1159,7 +1214,10 @@ class Uploader(service.MultiService):
                 uploader = AssistedUploader(self._helper)
             else:
                 uploader = self.uploader_class(self.parent)
-            self._all_uploads[uploader.get_upload_status()] = None
+            self._all_uploads[uploader] = None
+            self._recent_upload_status.append(uploader.get_upload_status())
+            while len(self._recent_upload_status) > self.MAX_UPLOAD_STATUSES:
+                self._recent_upload_status.pop(0)
             return uploader.start(uploadable)
         d.addCallback(_got_size)
         def _done(res):
@@ -1168,5 +1226,10 @@ class Uploader(service.MultiService):
         d.addBoth(_done)
         return d
 
-    def list_uploads(self):
+    def list_all_uploads(self):
         return self._all_uploads.keys()
+    def list_active_uploads(self):
+        return [u.get_upload_status() for u in self._all_uploads.keys()
+                if u.get_upload_status().get_active()]
+    def list_recent_uploads(self):
+        return self._recent_upload_status
