@@ -4,9 +4,9 @@ from twisted.application import service, strports, internet
 from twisted.web import static, resource, server, html, http
 from twisted.internet import defer, address
 from twisted.internet.interfaces import IConsumer
-from nevow import inevow, rend, loaders, appserver, url, tags as T
+from nevow import inevow, rend, appserver, url, tags as T
 from nevow.static import File as nevow_File # TODO: merge with static.File?
-from allmydata.util import base32, fileutil, idlib, observer, log
+from allmydata.util import fileutil, idlib, log
 import simplejson
 from allmydata.interfaces import IDownloadTarget, IDirectoryNode, IFileNode, \
      IMutableFileNode
@@ -22,38 +22,15 @@ from foolscap.eventual import fireEventually
 
 from nevow.util import resource_filename
 
-def getxmlfile(name):
-    return loaders.xmlfile(resource_filename('allmydata.web', '%s' % name))
+from allmydata.web import status, unlinked, introweb
+from allmydata.web.common import IClient, getxmlfile, get_arg, \
+     boolean_of_arg, abbreviate_size
 
-class IClient(Interface):
-    pass
 class ILocalAccess(Interface):
     def local_access_is_allowed():
         """Return True if t=upload&localdir= is allowed, giving anyone who
         can talk to the webserver control over the local (disk) filesystem."""
 
-def boolean_of_arg(arg):
-    assert arg.lower() in ("true", "t", "1", "false", "f", "0")
-    return arg.lower() in ("true", "t", "1")
-
-def get_arg(req, argname, default=None, multiple=False):
-    """Extract an argument from either the query args (req.args) or the form
-    body fields (req.fields). If multiple=False, this returns a single value
-    (or the default, which defaults to None), and the query args take
-    precedence. If multiple=True, this returns a tuple of arguments (possibly
-    empty), starting with all those in the query args.
-    """
-    results = []
-    if argname in req.args:
-        results.extend(req.args[argname])
-    if req.fields and argname in req.fields:
-        results.append(req.fields[argname].value)
-    if multiple:
-        return tuple(results)
-    if results:
-        return results[0]
-    return default
-
 # we must override twisted.web.http.Request.requestReceived with a version
 # that doesn't use cgi.parse_multipart() . Since we actually use Nevow, we
 # override the nevow-specific subclass, nevow.appserver.NevowRequest . This
@@ -797,6 +774,118 @@ class POSTHandler(rend.Page):
         d.addCallback(_got)
         return d
 
+    def _POST_mkdir(self, name):
+        d = self._check_replacement(name)
+        d.addCallback(lambda res: self._node.create_empty_directory(name))
+        d.addCallback(lambda res: "directory created")
+        return d
+
+    def _POST_uri(self, name, newuri):
+        d = self._check_replacement(name)
+        d.addCallback(lambda res: self._node.set_uri(name, newuri))
+        d.addCallback(lambda res: newuri)
+        return d
+
+    def _POST_delete(self, name):
+        if name is None:
+            # apparently an <input type="hidden" name="name" value="">
+            # won't show up in the resulting encoded form.. the 'name'
+            # field is completely missing. So to allow deletion of an
+            # empty file, we have to pretend that None means ''. The only
+            # downide of this is a slightly confusing error message if
+            # someone does a POST without a name= field. For our own HTML
+            # thisn't a big deal, because we create the 'delete' POST
+            # buttons ourselves.
+            name = ''
+        d = self._node.delete(name)
+        d.addCallback(lambda res: "thing deleted")
+        return d
+
+    def _POST_rename(self, name, from_name, to_name):
+        d = self._check_replacement(to_name)
+        d.addCallback(lambda res: self._node.get(from_name))
+        def add_dest(child):
+            uri = child.get_uri()
+            # now actually do the rename
+            return self._node.set_uri(to_name, uri)
+        d.addCallback(add_dest)
+        def rm_src(junk):
+            return self._node.delete(from_name)
+        d.addCallback(rm_src)
+        d.addCallback(lambda res: "thing renamed")
+        return d
+
+    def _POST_upload(self, contents, name, mutable, client):
+        if mutable:
+            # SDMF: files are small, and we can only upload data.
+            contents.file.seek(0)
+            data = contents.file.read()
+            #uploadable = FileHandle(contents.file)
+            d = self._check_replacement(name)
+            d.addCallback(lambda res: self._node.has_child(name))
+            def _checked(present):
+                if present:
+                    # modify the existing one instead of creating a new
+                    # one
+                    d2 = self._node.get(name)
+                    def _got_newnode(newnode):
+                        d3 = newnode.overwrite(data)
+                        d3.addCallback(lambda res: newnode.get_uri())
+                        return d3
+                    d2.addCallback(_got_newnode)
+                else:
+                    d2 = client.create_mutable_file(data)
+                    def _uploaded(newnode):
+                        d1 = self._node.set_node(name, newnode)
+                        d1.addCallback(lambda res: newnode.get_uri())
+                        return d1
+                    d2.addCallback(_uploaded)
+                return d2
+            d.addCallback(_checked)
+        else:
+            uploadable = FileHandle(contents.file)
+            d = self._check_replacement(name)
+            d.addCallback(lambda res: self._node.add_file(name, uploadable))
+            def _done(newnode):
+                return newnode.get_uri()
+            d.addCallback(_done)
+        return d
+
+    def _POST_overwrite(self, contents):
+        # SDMF: files are small, and we can only upload data.
+        contents.file.seek(0)
+        data = contents.file.read()
+        # TODO: 'name' handling needs review
+        d = defer.succeed(self._node)
+        def _got_child_overwrite(child_node):
+            child_node.overwrite(data)
+            return child_node.get_uri()
+        d.addCallback(_got_child_overwrite)
+        return d
+
+    def _POST_check(self, name):
+        d = self._node.get(name)
+        def _got_child_check(child_node):
+            d2 = child_node.check()
+            def _done(res):
+                log.msg("checked %s, results %s" % (child_node, res),
+                        facility="tahoe.webish", level=log.NOISY)
+                return str(res)
+            d2.addCallback(_done)
+            return d2
+        d.addCallback(_got_child_check)
+        return d
+
+    def _POST_set_children(self, children):
+        cs = []
+        for name, (file_or_dir, mddict) in children.iteritems():
+            cap = str(mddict.get('rw_uri') or mddict.get('ro_uri'))
+            cs.append((name, cap, mddict.get('metadata')))
+
+        d = self._node.set_children(cs)
+        d.addCallback(lambda res: "Okay so I did it.")
+        return d
+
     def renderHTTP(self, ctx):
         req = inevow.IRequest(ctx)
 
@@ -824,30 +913,15 @@ class POSTHandler(rend.Page):
         if t == "mkdir":
             if not name:
                 raise RuntimeError("mkdir requires a name")
-            d = self._check_replacement(name)
-            d.addCallback(lambda res: self._node.create_empty_directory(name))
-            d.addCallback(lambda res: "directory created")
+            d = self._POST_mkdir(name)
         elif t == "uri":
             if not name:
                 raise RuntimeError("set-uri requires a name")
             newuri = get_arg(req, "uri")
             assert newuri is not None
-            d = self._check_replacement(name)
-            d.addCallback(lambda res: self._node.set_uri(name, newuri))
-            d.addCallback(lambda res: newuri)
+            d = self._POST_uri(name, newuri)
         elif t == "delete":
-            if name is None:
-                # apparently an <input type="hidden" name="name" value="">
-                # won't show up in the resulting encoded form.. the 'name'
-                # field is completely missing. So to allow deletion of an
-                # empty file, we have to pretend that None means ''. The only
-                # downide of this is a slightly confusing error message if
-                # someone does a POST without a name= field. For our own HTML
-                # thisn't a big deal, because we create the 'delete' POST
-                # buttons ourselves.
-                name = ''
-            d = self._node.delete(name)
-            d.addCallback(lambda res: "thing deleted")
+            d = self._POST_delete(name)
         elif t == "rename":
             from_name = get_arg(req, "from_name")
             if from_name is not None:
@@ -868,18 +942,7 @@ class POSTHandler(rend.Page):
                     req.setResponseCode(http.BAD_REQUEST)
                     req.setHeader("content-type", "text/plain")
                     return "%s= may not contain a slash" % (k,)
-            d = self._check_replacement(to_name)
-            d.addCallback(lambda res: self._node.get(from_name))
-            def add_dest(child):
-                uri = child.get_uri()
-                # now actually do the rename
-                return self._node.set_uri(to_name, uri)
-            d.addCallback(add_dest)
-            def rm_src(junk):
-                return self._node.delete(from_name)
-            d.addCallback(rm_src)
-            d.addCallback(lambda res: "thing renamed")
-
+            d = self._POST_rename(name, from_name, to_name)
         elif t == "upload":
             contents = req.fields["file"]
             name = name or contents.filename
@@ -890,64 +953,23 @@ class POSTHandler(rend.Page):
                 raise RuntimeError("upload requires a name")
             name = name.decode(charset)
             assert isinstance(name, unicode)
-
-            if "mutable" in req.fields:
-                # SDMF: files are small, and we can only upload data.
-                contents.file.seek(0)
-                data = contents.file.read()
-                #uploadable = FileHandle(contents.file)
-                d = self._check_replacement(name)
-                d.addCallback(lambda res: self._node.has_child(name))
-                def _checked(present):
-                    if present:
-                        # modify the existing one instead of creating a new
-                        # one
-                        d2 = self._node.get(name)
-                        def _got_newnode(newnode):
-                            d3 = newnode.replace(data)
-                            d3.addCallback(lambda res: newnode.get_uri())
-                            return d3
-                        d2.addCallback(_got_newnode)
-                    else:
-                        d2 = IClient(ctx).create_mutable_file(data)
-                        def _uploaded(newnode):
-                            d1 = self._node.set_node(name, newnode)
-                            d1.addCallback(lambda res: newnode.get_uri())
-                            return d1
-                        d2.addCallback(_uploaded)
-                    return d2
-                d.addCallback(_checked)
-            else:
-                uploadable = FileHandle(contents.file)
-                d = self._check_replacement(name)
-                d.addCallback(lambda res: self._node.add_file(name, uploadable))
-                def _done(newnode):
-                    return newnode.get_uri()
-                d.addCallback(_done)
-
+            mutable = boolean_of_arg(get_arg(req, "mutable", "false"))
+            d = self._POST_upload(contents, name, mutable, IClient(ctx))
         elif t == "overwrite":
             contents = req.fields["file"]
-            # SDMF: files are small, and we can only upload data.
-            contents.file.seek(0)
-            data = contents.file.read()
-            # TODO: 'name' handling needs review
-            d = defer.succeed(self._node)
-            def _got_child_overwrite(child_node):
-                child_node.replace(data)
-                return child_node.get_uri()
-            d.addCallback(_got_child_overwrite)
-
+            d = self._POST_overwrite(contents)
         elif t == "check":
-            d = self._node.get(name)
-            def _got_child_check(child_node):
-                d2 = child_node.check()
-                def _done(res):
-                    log.msg("checked %s, results %s" % (child_node, res),
-                            facility="tahoe.webish", level=log.NOISY)
-                    return str(res)
-                d2.addCallback(_done)
-                return d2
-            d.addCallback(_got_child_check)
+            d = self._POST_check(name)
+        elif t == "set_children":
+            req.content.seek(0)
+            body = req.content.read()
+            try:
+                children = simplejson.loads(body)
+            except ValueError, le:
+                le.args = tuple(le.args + (body,))
+                # TODO test handling of bad JSON
+                raise
+            d = self._POST_set_children(children)
         else:
             print "BAD t=%s" % t
             return "BAD t=%s" % t
@@ -1293,317 +1315,6 @@ class VDrive(rend.Page):
             return rend.NotFound
         return d
 
-class UnlinkedPUTCHKUploader(rend.Page):
-    def renderHTTP(self, ctx):
-        req = inevow.IRequest(ctx)
-        assert req.method == "PUT"
-        # "PUT /uri", to create an unlinked file. This is like PUT but
-        # without the associated set_uri.
-
-        uploadable = FileHandle(req.content)
-        d = IClient(ctx).upload(uploadable)
-        d.addCallback(lambda results: results.uri)
-        # that fires with the URI of the new file
-        return d
-
-class UnlinkedPUTSSKUploader(rend.Page):
-    def renderHTTP(self, ctx):
-        req = inevow.IRequest(ctx)
-        assert req.method == "PUT"
-        # SDMF: files are small, and we can only upload data
-        contents = req.content
-        contents.seek(0)
-        data = contents.read()
-        d = IClient(ctx).create_mutable_file(data)
-        d.addCallback(lambda n: n.get_uri())
-        return d
-
-class UnlinkedPUTCreateDirectory(rend.Page):
-    def renderHTTP(self, ctx):
-        req = inevow.IRequest(ctx)
-        assert req.method == "PUT"
-        # "PUT /uri?t=mkdir", to create an unlinked directory.
-        d = IClient(ctx).create_empty_dirnode()
-        d.addCallback(lambda dirnode: dirnode.get_uri())
-        # XXX add redirect_to_result
-        return d
-
-
-class UnlinkedPOSTCHKUploader(rend.Page):
-    """'POST /uri', to create an unlinked file."""
-    docFactory = getxmlfile("unlinked-upload.xhtml")
-
-    def __init__(self, client, req):
-        rend.Page.__init__(self)
-        # we start the upload now, and distribute notification of its
-        # completion to render_ methods with an ObserverList
-        assert req.method == "POST"
-        self._done = observer.OneShotObserverList()
-        fileobj = req.fields["file"].file
-        uploadable = FileHandle(fileobj)
-        d = client.upload(uploadable)
-        d.addBoth(self._done.fire)
-
-    def renderHTTP(self, ctx):
-        req = inevow.IRequest(ctx)
-        when_done = get_arg(req, "when_done", None)
-        if when_done:
-            # if when_done= is provided, return a redirect instead of our
-            # usual upload-results page
-            d = self._done.when_fired()
-            d.addCallback(lambda res: url.URL.fromString(when_done))
-            return d
-        return rend.Page.renderHTTP(self, ctx)
-
-    def upload_results(self):
-        return self._done.when_fired()
-
-    def data_done(self, ctx, data):
-        d = self.upload_results()
-        d.addCallback(lambda res: "done!")
-        return d
-
-    def data_uri(self, ctx, data):
-        d = self.upload_results()
-        d.addCallback(lambda res: res.uri)
-        return d
-
-    def render_download_link(self, ctx, data):
-        d = self.upload_results()
-        d.addCallback(lambda res: T.a(href="/uri/" + urllib.quote(res.uri))
-                      ["/uri/" + res.uri])
-        return d
-
-    def render_sharemap(self, ctx, data):
-        d = self.upload_results()
-        d.addCallback(lambda res: res.sharemap)
-        def _render(sharemap):
-            if sharemap is None:
-                return "None"
-            l = T.ul()
-            for shnum in sorted(sharemap.keys()):
-                l[T.li["%d -> %s" % (shnum, sharemap[shnum])]]
-            return l
-        d.addCallback(_render)
-        return d
-
-    def render_servermap(self, ctx, data):
-        d = self.upload_results()
-        d.addCallback(lambda res: res.servermap)
-        def _render(servermap):
-            if servermap is None:
-                return "None"
-            l = T.ul()
-            for peerid in sorted(servermap.keys()):
-                peerid_s = idlib.shortnodeid_b2a(peerid)
-                shares_s = ",".join([str(shnum) for shnum in servermap[peerid]])
-                l[T.li["[%s] got shares: %s" % (peerid_s, shares_s)]]
-            return l
-        d.addCallback(_render)
-        return d
-
-    def data_file_size(self, ctx, data):
-        d = self.upload_results()
-        d.addCallback(lambda res: res.file_size)
-        return d
-
-    def render_time(self, ctx, data):
-        # 1.23s, 790ms, 132us
-        if data is None:
-            return ""
-        s = float(data)
-        if s >= 1.0:
-            return "%.2fs" % s
-        if s >= 0.01:
-            return "%dms" % (1000*s)
-        if s >= 0.001:
-            return "%.1fms" % (1000*s)
-        return "%dus" % (1000000*s)
-
-    def render_rate(self, ctx, data):
-        # 21.8kBps, 554.4kBps 4.37MBps
-        if data is None:
-            return ""
-        r = float(data)
-        if r > 1000000:
-            return "%1.2fMBps" % (r/1000000)
-        if r > 1000:
-            return "%.1fkBps" % (r/1000)
-        return "%dBps" % r
-
-    def _get_time(self, name):
-        d = self.upload_results()
-        d.addCallback(lambda res: res.timings.get(name))
-        return d
-
-    def data_time_total(self, ctx, data):
-        return self._get_time("total")
-
-    def data_time_storage_index(self, ctx, data):
-        return self._get_time("storage_index")
-
-    def data_time_contacting_helper(self, ctx, data):
-        return self._get_time("contacting_helper")
-
-    def data_time_existence_check(self, ctx, data):
-        return self._get_time("existence_check")
-
-    def data_time_cumulative_fetch(self, ctx, data):
-        return self._get_time("cumulative_fetch")
-
-    def data_time_helper_total(self, ctx, data):
-        return self._get_time("helper_total")
-
-    def data_time_peer_selection(self, ctx, data):
-        return self._get_time("peer_selection")
-
-    def data_time_total_encode_and_push(self, ctx, data):
-        return self._get_time("total_encode_and_push")
-
-    def data_time_cumulative_encoding(self, ctx, data):
-        return self._get_time("cumulative_encoding")
-
-    def data_time_cumulative_sending(self, ctx, data):
-        return self._get_time("cumulative_sending")
-
-    def data_time_hashes_and_close(self, ctx, data):
-        return self._get_time("hashes_and_close")
-
-    def _get_rate(self, name):
-        d = self.upload_results()
-        def _convert(r):
-            file_size = r.file_size
-            time = r.timings.get(name)
-            if time is None:
-                return None
-            try:
-                return 1.0 * file_size / time
-            except ZeroDivisionError:
-                return None
-        d.addCallback(_convert)
-        return d
-
-    def data_rate_total(self, ctx, data):
-        return self._get_rate("total")
-
-    def data_rate_storage_index(self, ctx, data):
-        return self._get_rate("storage_index")
-
-    def data_rate_encode(self, ctx, data):
-        return self._get_rate("cumulative_encoding")
-
-    def data_rate_push(self, ctx, data):
-        return self._get_rate("cumulative_sending")
-
-    def data_rate_encode_and_push(self, ctx, data):
-        d = self.upload_results()
-        def _convert(r):
-            file_size = r.file_size
-            if file_size is None:
-                return None
-            time1 = r.timings.get("cumulative_encoding")
-            if time1 is None:
-                return None
-            time2 = r.timings.get("cumulative_sending")
-            if time2 is None:
-                return None
-            try:
-                return 1.0 * file_size / (time1+time2)
-            except ZeroDivisionError:
-                return None
-        d.addCallback(_convert)
-        return d
-
-    def data_rate_ciphertext_fetch(self, ctx, data):
-        d = self.upload_results()
-        def _convert(r):
-            fetch_size = r.ciphertext_fetched
-            if fetch_size is None:
-                return None
-            time = r.timings.get("cumulative_fetch")
-            if time is None:
-                return None
-            try:
-                return 1.0 * fetch_size / time
-            except ZeroDivisionError:
-                return None
-        d.addCallback(_convert)
-        return d
-
-class UnlinkedPOSTSSKUploader(rend.Page):
-    def renderHTTP(self, ctx):
-        req = inevow.IRequest(ctx)
-        assert req.method == "POST"
-
-        # "POST /uri", to create an unlinked file.
-        # SDMF: files are small, and we can only upload data
-        contents = req.fields["file"]
-        contents.file.seek(0)
-        data = contents.file.read()
-        d = IClient(ctx).create_mutable_file(data)
-        d.addCallback(lambda n: n.get_uri())
-        return d
-
-class UnlinkedPOSTCreateDirectory(rend.Page):
-    def renderHTTP(self, ctx):
-        req = inevow.IRequest(ctx)
-        assert req.method == "POST"
-
-        # "POST /uri?t=mkdir", to create an unlinked directory.
-        d = IClient(ctx).create_empty_dirnode()
-        redirect = get_arg(req, "redirect_to_result", "false")
-        if boolean_of_arg(redirect):
-            def _then_redir(res):
-                new_url = "uri/" + urllib.quote(res.get_uri())
-                req.setResponseCode(http.SEE_OTHER) # 303
-                req.setHeader('location', new_url)
-                req.finish()
-                return ''
-            d.addCallback(_then_redir)
-        else:
-            d.addCallback(lambda dirnode: dirnode.get_uri())
-        return d
-
-class Status(rend.Page):
-    docFactory = getxmlfile("status.xhtml")
-
-    def data_uploads(self, ctx, data):
-        return IClient(ctx).list_uploads()
-
-    def data_downloads(self, ctx, data):
-        return IClient(ctx).list_downloads()
-
-    def _render_common(self, ctx, data):
-        s = data
-        si_s = base32.b2a_or_none(s.get_storage_index())
-        if si_s is None:
-            si_s = "(None)"
-        ctx.fillSlots("si", si_s)
-        ctx.fillSlots("helper", {True: "Yes",
-                                 False: "No"}[s.using_helper()])
-        size = s.get_size()
-        if size is None:
-            size = "(unknown)"
-        ctx.fillSlots("total_size", size)
-        ctx.fillSlots("status", s.get_status())
-
-    def render_row_upload(self, ctx, data):
-        self._render_common(ctx, data)
-        (chk, ciphertext, encandpush) = data.get_progress()
-        # TODO: make an ascii-art bar
-        ctx.fillSlots("progress_hash", "%.1f%%" % (100.0 * chk))
-        ctx.fillSlots("progress_ciphertext", "%.1f%%" % (100.0 * ciphertext))
-        ctx.fillSlots("progress_encode", "%.1f%%" % (100.0 * encandpush))
-        return ctx.tag
-
-    def render_row_download(self, ctx, data):
-        self._render_common(ctx, data)
-        progress = data.get_progress()
-        # TODO: make an ascii-art bar
-        ctx.fillSlots("progress", "%.1f%%" % (100.0 * progress))
-        return ctx.tag
-
-
 class Root(rend.Page):
 
     addSlash = True
@@ -1637,11 +1348,11 @@ class Root(rend.Page):
                         if t == "":
                             mutable = bool(get_arg(req, "mutable", "").strip())
                             if mutable:
-                                return UnlinkedPUTSSKUploader(), ()
+                                return unlinked.UnlinkedPUTSSKUploader(), ()
                             else:
-                                return UnlinkedPUTCHKUploader(), ()
+                                return unlinked.UnlinkedPUTCHKUploader(), ()
                         if t == "mkdir":
-                            return UnlinkedPUTCreateDirectory(), ()
+                            return unlinked.UnlinkedPUTCreateDirectory(), ()
                         errmsg = "/uri only accepts PUT and PUT?t=mkdir"
                         return WebError(http.BAD_REQUEST, errmsg), ()
 
@@ -1653,11 +1364,11 @@ class Root(rend.Page):
                         if t in ("", "upload"):
                             mutable = bool(get_arg(req, "mutable", "").strip())
                             if mutable:
-                                return UnlinkedPOSTSSKUploader(), ()
+                                return unlinked.UnlinkedPOSTSSKUploader(), ()
                             else:
-                                return UnlinkedPOSTCHKUploader(client, req), ()
+                                return unlinked.UnlinkedPOSTCHKUploader(client, req), ()
                         if t == "mkdir":
-                            return UnlinkedPOSTCreateDirectory(), ()
+                            return unlinked.UnlinkedPOSTCreateDirectory(), ()
                         errmsg = "/uri accepts only PUT, PUT?t=mkdir, POST?t=upload, and POST?t=mkdir"
                         return WebError(http.BAD_REQUEST, errmsg), ()
                 if len(segments) < 2:
@@ -1679,7 +1390,7 @@ class Root(rend.Page):
     child_tahoe_css = nevow_File(resource_filename('allmydata.web', 'tahoe.css'))
 
     child_provisioning = provisioning.ProvisioningTool()
-    child_status = Status()
+    child_status = status.Status()
 
     def data_version(self, ctx, data):
         return get_package_versions_string()
@@ -1693,8 +1404,11 @@ class Root(rend.Page):
             ss = client.getServiceNamed("storage")
         except KeyError:
             return "Not running"
-        allocated = ss.allocated_size()
-        return "about %d bytes allocated" % allocated
+        allocated = "about %s allocated" % abbreviate_size(ss.allocated_size())
+        sizelimit = "no size limit"
+        if ss.sizelimit is not None:
+            sizelimit = "size limit is %s" % abbreviate_size(ss.sizelimit)
+        return "%s, %s" % (allocated, sizelimit)
 
     def data_introducer_furl(self, ctx, data):
         return IClient(ctx).introducer_furl
@@ -1830,11 +1544,12 @@ class LocalAccess:
 
 class WebishServer(service.MultiService):
     name = "webish"
+    root_class = Root
 
     def __init__(self, webport, nodeurl_path=None):
         service.MultiService.__init__(self)
         self.webport = webport
-        self.root = Root()
+        self.root = self.root_class()
         self.site = site = appserver.NevowSite(self.root)
         self.site.requestFactory = MyRequest
         self.allow_local = LocalAccess()
@@ -1865,9 +1580,9 @@ class WebishServer(service.MultiService):
         # what is our webport?
         s = self.listener
         if isinstance(s, internet.TCPServer):
-            base_url = "http://localhost:%d" % s._port.getHost().port
+            base_url = "http://127.0.0.1:%d/" % s._port.getHost().port
         elif isinstance(s, internet.SSLServer):
-            base_url = "https://localhost:%d" % s._port.getHost().port
+            base_url = "https://127.0.0.1:%d/" % s._port.getHost().port
         else:
             base_url = None
         if base_url:
@@ -1876,3 +1591,5 @@ class WebishServer(service.MultiService):
             f.write(base_url + "\n")
             f.close()
 
+class IntroducerWebishServer(WebishServer):
+    root_class = introweb.IntroducerRoot
